@@ -24,11 +24,12 @@ import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
 
+import org.apache.cayenne.cache.NestedQueryCache;
+import org.apache.cayenne.configuration.CayenneRuntime;
 import org.apache.cayenne.event.EventManager;
 import org.apache.cayenne.graph.CompoundDiff;
 import org.apache.cayenne.graph.GraphDiff;
 import org.apache.cayenne.graph.GraphManager;
-import org.apache.cayenne.map.EntityResolver;
 import org.apache.cayenne.map.ObjEntity;
 import org.apache.cayenne.query.Query;
 import org.apache.cayenne.reflect.ClassDescriptor;
@@ -45,27 +46,8 @@ import org.apache.cayenne.validation.ValidationResult;
  */
 public class CayenneContext extends BaseContext {
 
-    /**
-     * @since 3.0
-     */
-    private static ThreadLocal<PropertyChangeProcessingStrategy> PROPERTY_CHANGE_PROCESSING_STRATEGY = new ThreadLocal<PropertyChangeProcessingStrategy>() {
-
-        @Override
-        protected PropertyChangeProcessingStrategy initialValue() {
-            return PropertyChangeProcessingStrategy.RECORD_AND_PROCESS_REVERSE_ARCS;
-        }
-    };
-
-    protected EntityResolver entityResolver;
-
     CayenneContextGraphManager graphManager;
 
-    // note that it is important to reuse the same action within the property change
-    // thread to avoid a loop of "propertyChange" calls on handling reverse relationships.
-    // Here we go further and make action a thread-safe ivar that tracks its own thread
-    // state.
-    CayenneContextGraphAction graphAction;
-
     // object that merges "backdoor" changes that come from the channel.
     CayenneContextMergeHandler mergeHandler;
 
@@ -86,64 +68,41 @@ public class CayenneContext extends BaseContext {
     }
 
     /**
-     * Creates a new CayenneContext, initializing it with a channel. If
-     * <code>graphEventsEnabled</code> is true, this context will broadcast GraphEvents
-     * using ObjectContext.GRAPH_CHANGE_SUBJECT.
+     * Creates a new CayenneContext, initializing it with a channel.
      */
     public CayenneContext(DataChannel channel, boolean changeEventsEnabled,
-            boolean syncEventsEnabled) {
+            boolean lifecyleEventsEnabled) {
 
-        this.graphAction = new CayenneContextGraphAction(this);
-        this.graphManager = new CayenneContextGraphManager(
+        graphManager = new CayenneContextGraphManager(
                 this,
                 changeEventsEnabled,
-                syncEventsEnabled);
+                lifecyleEventsEnabled);
 
-        setChannel(channel);
+        if (channel != null) {
+            attachToChannel(channel);
+        }
     }
 
     /**
-     * @since 3.0
+     * @since 3.1
      */
-    // accesses a static thread local variable... still keeping the method non-static for
-    // better future portability...
-    PropertyChangeProcessingStrategy getPropertyChangeProcessingStrategy() {
-        return PROPERTY_CHANGE_PROCESSING_STRATEGY.get();
-    }
+    @Override
+    protected void attachToChannel(DataChannel channel) {
+        super.attachToChannel(channel);
 
-    /**
-     * @since 3.0
-     */
-    // accesses a static thread local variable... still keeping the method non-static for
-    // better future portability...
-    void setPropertyChangeProcessingStrategy(PropertyChangeProcessingStrategy strategy) {
-        PROPERTY_CHANGE_PROCESSING_STRATEGY.set(strategy);
-    }
+        if (mergeHandler != null) {
+            mergeHandler.active = false;
+            mergeHandler = null;
+        }
 
-    /**
-     * Sets the context channel, setting up a listener for channel events.
-     */
-    public void setChannel(DataChannel channel) {
-        if (this.channel != channel) {
+        EventManager eventManager = channel.getEventManager();
+        if (eventManager != null) {
+            mergeHandler = new CayenneContextMergeHandler(this);
 
-            if (this.mergeHandler != null) {
-                this.mergeHandler.active = false;
-                this.mergeHandler = null;
-            }
-
-            this.channel = channel;
-
-            EventManager eventManager = (channel != null)
-                    ? channel.getEventManager()
-                    : null;
-            if (eventManager != null) {
-                this.mergeHandler = new CayenneContextMergeHandler(this);
-
-                // listen to our channel events...
-                // note that we must reset listener on channel switch, as there is no
-                // guarantee that a new channel uses the same EventManager.
-                EventUtil.listenForChannelEvents(channel, mergeHandler);
-            }
+            // listen to our channel events...
+            // note that we must reset listener on channel switch, as there is no
+            // guarantee that a new channel uses the same EventManager.
+            EventUtil.listenForChannelEvents(channel, mergeHandler);
         }
     }
 
@@ -166,29 +125,6 @@ public class CayenneContext extends BaseContext {
         return graphManager.lifecycleEventsEnabled;
     }
 
-    /**
-     * Returns an EntityResolver that provides mapping information needed for
-     * CayenneContext operation. If EntityResolver is not set, this method would obtain
-     * and cache one from the underlying DataChannel.
-     */
-    @Override
-    public EntityResolver getEntityResolver() {
-        // load entity resolver on demand
-        if (entityResolver == null) {
-            synchronized (this) {
-                if (entityResolver == null) {
-                    setEntityResolver(channel.getEntityResolver());
-                }
-            }
-        }
-
-        return entityResolver;
-    }
-
-    public void setEntityResolver(EntityResolver entityResolver) {
-        this.entityResolver = entityResolver;
-    }
-
     @Override
     public GraphManager getGraphManager() {
         return graphManager;
@@ -198,10 +134,6 @@ public class CayenneContext extends BaseContext {
         return graphManager;
     }
 
-    CayenneContextGraphAction internalGraphAction() {
-        return graphAction;
-    }
-
     /**
      * Commits changes to uncommitted objects. First checks if there are changes in this
      * context and if any changes are detected, sends a commit message to remote Cayenne
@@ -224,27 +156,29 @@ public class CayenneContext extends BaseContext {
 
             if (graphManager.hasChanges()) {
 
-                ValidationResult result = new ValidationResult();
-                Iterator<?> it = graphManager.dirtyNodes().iterator();
-                while (it.hasNext()) {
-                    Persistent p = (Persistent) it.next();
-                    if (p instanceof Validating) {
-                        switch (p.getPersistenceState()) {
-                            case PersistenceState.NEW:
-                                ((Validating) p).validateForInsert(result);
-                                break;
-                            case PersistenceState.MODIFIED:
-                                ((Validating) p).validateForUpdate(result);
-                                break;
-                            case PersistenceState.DELETED:
-                                ((Validating) p).validateForDelete(result);
-                                break;
+                if (isValidatingObjectsOnCommit()) {
+                    ValidationResult result = new ValidationResult();
+                    Iterator<?> it = graphManager.dirtyNodes().iterator();
+                    while (it.hasNext()) {
+                        Persistent p = (Persistent) it.next();
+                        if (p instanceof Validating) {
+                            switch (p.getPersistenceState()) {
+                                case PersistenceState.NEW:
+                                    ((Validating) p).validateForInsert(result);
+                                    break;
+                                case PersistenceState.MODIFIED:
+                                    ((Validating) p).validateForUpdate(result);
+                                    break;
+                                case PersistenceState.DELETED:
+                                    ((Validating) p).validateForDelete(result);
+                                    break;
+                            }
                         }
                     }
-                }
 
-                if (result.hasFailures()) {
-                    throw new ValidationException(result);
+                    if (result.hasFailures()) {
+                        throw new ValidationException(result);
+                    }
                 }
 
                 graphManager.graphCommitStarted();
@@ -307,27 +241,6 @@ public class CayenneContext extends BaseContext {
         }
     }
 
-    /**
-     * Deletes an object locally, scheduling it for future deletion from the external data
-     * store.
-     */
-    @Override
-    public void deleteObject(Object object) {
-        new ObjectContextDeleteAction(this).performDelete((Persistent) object);
-    }
-
-    public void deleteObjects(Collection<?> objects) throws DeleteDenyException {
-        if (objects.isEmpty())
-            return;
-
-        // Don't call deleteObject() directly since it would be less efficient.
-        ObjectContextDeleteAction ocda = new ObjectContextDeleteAction(this);
-
-        // Make a copy to iterate over to avoid ConcurrentModificationException.
-        for (Persistent object : (ArrayList<Persistent>) new ArrayList(objects))
-            ocda.performDelete(object);
-    }
-
     /**
      * Creates and registers a new Persistent object instance.
      */
@@ -346,6 +259,7 @@ public class CayenneContext extends BaseContext {
         ClassDescriptor descriptor = getEntityResolver().getClassDescriptor(
                 entity.getName());
         T object = (T) descriptor.createObject();
+        descriptor.injectValueHolders(object);
         registerNewObject((Persistent) object, entity.getName(), descriptor);
         return object;
     }
@@ -384,105 +298,6 @@ public class CayenneContext extends BaseContext {
         return new CayenneContextQueryAction(this, context, query).execute();
     }
 
-    /**
-     * Converts a list of Persistent objects registered in some other ObjectContext to a
-     * list of objects local to this ObjectContext.
-     * <p>
-     * <i>Current limitation: all objects in the source list must be either in COMMITTED
-     * or in HOLLOW state.</i>
-     * </p>
-     */
-    @Override
-    public Persistent localObject(ObjectId id, Object prototype) {
-
-        // TODO: Andrus, 1/26/2006 - this implementation is copied verbatim from
-        // DataContext. Somehow need to pull out the common code or implement inheritance
-
-        // ****** Copied from DataContext - start *******
-
-        if (id == null) {
-            throw new IllegalArgumentException("Null ObjectId");
-        }
-
-        ClassDescriptor descriptor = getEntityResolver().getClassDescriptor(
-                id.getEntityName());
-
-        synchronized (getGraphManager()) {
-            Persistent cachedObject = (Persistent) getGraphManager().getNode(id);
-
-            // merge into an existing object
-            if (cachedObject != null) {
-
-                // TODO: Andrus, 1/24/2006 implement smart merge for modified objects...
-                if (cachedObject != prototype
-                        && cachedObject.getPersistenceState() != PersistenceState.MODIFIED
-                        && cachedObject.getPersistenceState() != PersistenceState.DELETED) {
-
-                    if (prototype != null
-                            && ((Persistent) prototype).getPersistenceState() != PersistenceState.HOLLOW) {
-
-                        descriptor.shallowMerge(prototype, cachedObject);
-
-                        if (cachedObject.getPersistenceState() == PersistenceState.HOLLOW) {
-                            cachedObject.setPersistenceState(PersistenceState.COMMITTED);
-                        }
-                    }
-                }
-
-                return cachedObject;
-            }
-            // create and merge into a new object
-            else {
-
-                // Andrus, 1/26/2006 - note that there is a tricky case of a temporary
-                // object
-                // passed from peer DataContext... In the past we used to throw an
-                // exception
-                // or return null. Now that we can have a valid (but generally
-                // indistinguishible) case of such object passed from parent, we let it
-                // slip... Not sure what's the best way of handling it that does not
-                // involve
-                // breaking encapsulation of the DataChannel to detect where in the
-                // hierarchy
-                // this context is.
-
-                Persistent localObject;
-
-                localObject = (Persistent) descriptor.createObject();
-
-                localObject.setObjectContext(this);
-                localObject.setObjectId(id);
-
-                getGraphManager().registerNode(id, localObject);
-
-                if (prototype != null
-                        && ((Persistent) prototype).getPersistenceState() != PersistenceState.HOLLOW) {
-                    localObject.setPersistenceState(PersistenceState.COMMITTED);
-                    descriptor.shallowMerge(prototype, localObject);
-                }
-                else {
-                    localObject.setPersistenceState(PersistenceState.HOLLOW);
-                }
-
-                return localObject;
-            }
-        }
-
-        // ****** Copied from DataContext - end *******
-    }
-
-    @Override
-    public void propertyChanged(
-            Persistent object,
-            String property,
-            Object oldValue,
-            Object newValue) {
-
-        if (getPropertyChangeProcessingStrategy() != PropertyChangeProcessingStrategy.IGNORE) {
-            graphAction.handlePropertyChange(object, property, oldValue, newValue);
-        }
-    }
-
     @Override
     public Collection<?> uncommittedObjects() {
         synchronized (graphManager) {
@@ -527,16 +342,6 @@ public class CayenneContext extends BaseContext {
             object.setObjectId(id);
         }
 
-        // must follow this exact order of property initialization per CAY-653, i.e. have
-        // the id and the context in place BEFORE setPersistence is called
-        object.setObjectContext(this);
-        object.setPersistenceState(PersistenceState.NEW);
-        
-        synchronized (graphManager) {
-            graphManager.registerNode(id, object);
-            graphManager.nodeCreated(id);
-        }
-        
         injectInitialValue(object);
     }
 
@@ -562,12 +367,22 @@ public class CayenneContext extends BaseContext {
      * Creates and returns a new child ObjectContext.
      * 
      * @since 3.0
+     * @deprecated Since 3.1 replaced by {@link CayenneRuntime#getContext(DataChannel)}
      */
+    @Deprecated
     public ObjectContext createChildContext() {
-        return new CayenneContext(
+        CayenneContext child = new CayenneContext(
                 this,
                 graphManager.changeEventsEnabled,
                 graphManager.lifecycleEventsEnabled);
+
+        // TODO: This method should be deprecated and child context should be created via
+        // DI with all proper injection, so won't have to guess how to handle query cache.
+        if (queryCache != null) {
+            child.setQueryCache(new NestedQueryCache(queryCache));
+        }
+
+        return child;
     }
 
     @Override
@@ -580,15 +395,16 @@ public class CayenneContext extends BaseContext {
 
         if (childContext) {
 
-            PropertyChangeProcessingStrategy oldStrategy = getPropertyChangeProcessingStrategy();
-            setPropertyChangeProcessingStrategy(PropertyChangeProcessingStrategy.RECORD);
+            // PropertyChangeProcessingStrategy oldStrategy =
+            // getPropertyChangeProcessingStrategy();
+            // setPropertyChangeProcessingStrategy(PropertyChangeProcessingStrategy.RECORD);
             try {
                 changes.apply(new CayenneContextChildDiffLoader(this));
             }
             finally {
-                setPropertyChangeProcessingStrategy(oldStrategy);
+                // setPropertyChangeProcessingStrategy(oldStrategy);
             }
-            
+
             fireDataChannelChanged(originatingContext, changes);
         }
 
