@@ -19,6 +19,43 @@
 
 package org.apache.cayenne.access;
 
+import org.apache.cayenne.BaseContext;
+import org.apache.cayenne.CayenneException;
+import org.apache.cayenne.CayenneRuntimeException;
+import org.apache.cayenne.DataChannel;
+import org.apache.cayenne.DataObject;
+import org.apache.cayenne.DataRow;
+import org.apache.cayenne.Fault;
+import org.apache.cayenne.ObjectContext;
+import org.apache.cayenne.ObjectId;
+import org.apache.cayenne.PersistenceState;
+import org.apache.cayenne.Persistent;
+import org.apache.cayenne.QueryResponse;
+import org.apache.cayenne.access.util.IteratedSelectObserver;
+import org.apache.cayenne.cache.NestedQueryCache;
+import org.apache.cayenne.configuration.CayenneRuntime;
+import org.apache.cayenne.event.EventManager;
+import org.apache.cayenne.graph.ChildDiffLoader;
+import org.apache.cayenne.graph.CompoundDiff;
+import org.apache.cayenne.graph.GraphDiff;
+import org.apache.cayenne.graph.GraphManager;
+import org.apache.cayenne.map.DbJoin;
+import org.apache.cayenne.map.DbRelationship;
+import org.apache.cayenne.map.ObjAttribute;
+import org.apache.cayenne.map.ObjEntity;
+import org.apache.cayenne.map.ObjRelationship;
+import org.apache.cayenne.query.NamedQuery;
+import org.apache.cayenne.query.Query;
+import org.apache.cayenne.query.QueryMetadata;
+import org.apache.cayenne.reflect.AttributeProperty;
+import org.apache.cayenne.reflect.ClassDescriptor;
+import org.apache.cayenne.reflect.PropertyVisitor;
+import org.apache.cayenne.reflect.ToManyProperty;
+import org.apache.cayenne.reflect.ToOneProperty;
+import org.apache.cayenne.util.EventUtil;
+import org.apache.cayenne.util.GenericResponse;
+import org.apache.cayenne.util.Util;
+
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
@@ -29,163 +66,20 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
-import org.apache.cayenne.BaseContext;
-import org.apache.cayenne.CayenneException;
-import org.apache.cayenne.CayenneRuntimeException;
-import org.apache.cayenne.DataChannel;
-import org.apache.cayenne.DataObject;
-import org.apache.cayenne.DataObjectUtils;
-import org.apache.cayenne.DataRow;
-import org.apache.cayenne.DeleteDenyException;
-import org.apache.cayenne.Fault;
-import org.apache.cayenne.ObjectContext;
-import org.apache.cayenne.ObjectId;
-import org.apache.cayenne.PersistenceState;
-import org.apache.cayenne.Persistent;
-import org.apache.cayenne.QueryResponse;
-import org.apache.cayenne.access.util.IteratedSelectObserver;
-import org.apache.cayenne.cache.QueryCache;
-import org.apache.cayenne.cache.QueryCacheFactory;
-import org.apache.cayenne.conf.Configuration;
-import org.apache.cayenne.event.EventManager;
-import org.apache.cayenne.graph.ChildDiffLoader;
-import org.apache.cayenne.graph.CompoundDiff;
-import org.apache.cayenne.graph.GraphDiff;
-import org.apache.cayenne.graph.GraphManager;
-import org.apache.cayenne.map.DbJoin;
-import org.apache.cayenne.map.DbRelationship;
-import org.apache.cayenne.map.EntityResolver;
-import org.apache.cayenne.map.LifecycleEvent;
-import org.apache.cayenne.map.ObjAttribute;
-import org.apache.cayenne.map.ObjEntity;
-import org.apache.cayenne.map.ObjRelationship;
-import org.apache.cayenne.query.NamedQuery;
-import org.apache.cayenne.query.ObjectIdQuery;
-import org.apache.cayenne.query.Query;
-import org.apache.cayenne.query.QueryMetadata;
-import org.apache.cayenne.reflect.AttributeProperty;
-import org.apache.cayenne.reflect.ClassDescriptor;
-import org.apache.cayenne.reflect.PropertyVisitor;
-import org.apache.cayenne.reflect.ToManyProperty;
-import org.apache.cayenne.reflect.ToOneProperty;
-import org.apache.cayenne.util.EventUtil;
-import org.apache.cayenne.util.GenericResponse;
-import org.apache.cayenne.util.ObjectContextGraphAction;
-import org.apache.cayenne.util.Util;
-
 /**
  * The most common implementation of {@link ObjectContext}. DataContext is an isolated
  * container of an object graph, in a sense that any uncommitted changes to persistent
  * objects that are registered with the context, are not visible to the users of other
  * contexts.
  */
-public class DataContext extends BaseContext implements DataChannel {
+public class DataContext extends BaseContext {
 
-    // Set of DataContextDelegates to be notified.
     private DataContextDelegate delegate;
-
     protected boolean usingSharedSnaphsotCache;
-    protected boolean validatingObjectsOnCommit;
     protected ObjectStore objectStore;
 
-    // note that entity resolver is initialized from the parent channel the first time it
-    // is accessed, and later cached in the context
-    protected transient EntityResolver entityResolver;
-
     protected transient DataContextMergeHandler mergeHandler;
 
-    ObjectContextGraphAction graphAction;
-
-    /**
-     * Stores the name of parent DataDomain. Used to defer initialization of the parent
-     * QueryEngine after deserialization. This helps avoid an issue with certain servlet
-     * engines (e.g. Tomcat) where HttpSessions with DataContext's are deserialized at
-     * startup before Cayenne stack is fully initialized.
-     */
-    protected transient String lazyInitParentDomainName;
-
-    /**
-     * Returns the DataContext bound to the current thread.
-     * 
-     * @since 1.1
-     * @return the DataContext associated with caller thread.
-     * @throws IllegalStateException if there is no DataContext bound to the current
-     *             thread.
-     * @see org.apache.cayenne.conf.WebApplicationContextFilter
-     * @deprecated since 3.0, replaced by BaseContex#getThreadObjectContext().
-     */
-    @Deprecated
-    public static DataContext getThreadDataContext() throws IllegalStateException {
-        return (DataContext) BaseContext.getThreadObjectContext();
-    }
-
-    /**
-     * Binds a DataContext to the current thread. DataContext can later be retrieved by
-     * users in the same thread by calling {@link DataContext#getThreadDataContext}. Using
-     * null parameter will unbind currently bound DataContext.
-     * 
-     * @since 1.1
-     * @deprecated since 3.0, replaced by BaseContex#bindThreadObjectContext().
-     */
-    @Deprecated
-    public static void bindThreadDataContext(DataContext context) {
-        BaseContext.bindThreadObjectContext(context);
-    }
-
-    /**
-     * Factory method that creates and returns a new instance of DataContext based on
-     * default domain. If more than one domain exists in the current configuration,
-     * {@link DataContext#createDataContext(String)} must be used instead. ObjectStore
-     * associated with created DataContext will have a cache stack configured using parent
-     * domain settings.
-     */
-    public static DataContext createDataContext() {
-        return Configuration.getSharedConfiguration().getDomain().createDataContext();
-    }
-
-    /**
-     * Factory method that creates and returns a new instance of DataContext based on
-     * default domain. If more than one domain exists in the current configuration,
-     * {@link DataContext#createDataContext(String, boolean)} must be used instead.
-     * ObjectStore associated with newly created DataContext will have a cache stack
-     * configured according to the specified policy, overriding a parent domain setting.
-     * 
-     * @since 1.1
-     */
-    public static DataContext createDataContext(boolean useSharedCache) {
-        return Configuration.getSharedConfiguration().getDomain().createDataContext(
-                useSharedCache);
-    }
-
-    /**
-     * Factory method that creates and returns a new instance of DataContext using named
-     * domain as its parent. If there is no domain matching the name argument, an
-     * exception is thrown.
-     */
-    public static DataContext createDataContext(String domainName) {
-        DataDomain domain = Configuration.getSharedConfiguration().getDomain(domainName);
-        if (domain == null) {
-            throw new IllegalArgumentException("Non-existent domain: " + domainName);
-        }
-        return domain.createDataContext();
-    }
-
-    /**
-     * Creates and returns new DataContext that will use a named DataDomain as its parent.
-     * ObjectStore associated with newly created DataContext will have a cache stack
-     * configured according to the specified policy, overriding a parent domain setting.
-     * 
-     * @since 1.1
-     */
-    public static DataContext createDataContext(String domainName, boolean useSharedCache) {
-
-        DataDomain domain = Configuration.getSharedConfiguration().getDomain(domainName);
-        if (domain == null) {
-            throw new IllegalArgumentException("Non-existent domain: " + domainName);
-        }
-        return domain.createDataContext(useSharedCache);
-    }
-
     /**
      * Creates a new DataContext that is not attached to the Cayenne stack.
      */
@@ -199,71 +93,44 @@ public class DataContext extends BaseContext implements DataChannel {
      * @since 1.2
      */
     public DataContext(DataChannel channel, ObjectStore objectStore) {
-        // use a setter to properly initialize EntityResolver
-        setChannel(channel);
 
         // inject self as parent context
         if (objectStore != null) {
             this.objectStore = objectStore;
             objectStore.setContext(this);
+        }
 
+        if (channel != null) {
+            attachToChannel(channel);
+        }
+
+        if (objectStore != null) {
             DataDomain domain = getParentDataDomain();
             this.usingSharedSnaphsotCache = domain != null
                     && objectStore.getDataRowCache() == domain.getSharedSnapshotCache();
         }
-
-        this.graphAction = new DataContextGraphAction(this);
-    }
-
-    /**
-     * Returns {@link QueryCache} used by this DataContext, creating it on the fly if
-     * needed. Uses parent DataDomain {@link QueryCacheFactory} to initialize the cache
-     * for the first time, passing parent DataDomain's properties.
-     * 
-     * @since 3.0
-     */
-    @Override
-    public QueryCache getQueryCache() {
-        if (queryCache == null) {
-            synchronized (this) {
-                if (queryCache == null) {
-
-                    DataDomain domain = getParentDataDomain();
-                    queryCache = domain.getQueryCacheFactory().getQueryCache(
-                            domain.getProperties());
-                }
-            }
-        }
-
-        return queryCache;
     }
 
     /**
      * Creates and returns a new child ObjectContext.
      * 
      * @since 3.0
-     */
-    public ObjectContext createChildContext() {
-        return createChildDataContext();
-    }
-
-    /**
-     * Creates and returns a new child DataContext.
-     * 
-     * @since 1.2
-     * @deprecated since 3.0 use {@link #createChildContext()}.
+     * @deprecated Since 3.1 replaced by {@link CayenneRuntime#getContext(DataChannel)}
      */
     @Deprecated
-    public DataContext createChildDataContext() {
-        DataContextFactory factory = getParentDataDomain().getDataContextFactory();
+    public ObjectContext createChildContext() {
 
         // child ObjectStore should not have direct access to snapshot cache, so do not
         // pass it in constructor.
         ObjectStore objectStore = new ObjectStore();
 
-        DataContext child = factory != null
-                ? factory.createDataContext(this, objectStore)
-                : new DataContext(this, objectStore);
+        DataContext child = new DataContext(this, objectStore);
+
+        // TODO: This method should be deprecated and child context should be created via
+        // DI with all proper injection, so won't have to guess how to handle query cache.
+        if (queryCache != null) {
+            child.setQueryCache(new NestedQueryCache(queryCache));
+        }
 
         child.setValidatingObjectsOnCommit(isValidatingObjectsOnCommit());
         child.usingSharedSnaphsotCache = isUsingSharedSnapshotCache();
@@ -271,54 +138,38 @@ public class DataContext extends BaseContext implements DataChannel {
     }
 
     /**
-     * @since 1.2
+     * @since 3.1
      */
-    public void setChannel(DataChannel channel) {
-        if (this.channel != channel) {
+    @Override
+    protected void attachToChannel(DataChannel channel) {
 
-            if (this.mergeHandler != null) {
-                this.mergeHandler.setActive(false);
-            }
+        super.attachToChannel(channel);
 
-            this.entityResolver = null;
-            this.mergeHandler = null;
+        if (mergeHandler != null) {
+            mergeHandler.setActive(false);
+            mergeHandler = null;
+        }
 
-            this.channel = channel;
+        EventManager eventManager = channel.getEventManager();
 
-            if (channel != null) {
+        if (eventManager != null) {
+            mergeHandler = new DataContextMergeHandler(this);
 
-                // cache entity resolver, as we have no idea how expensive it is to query
-                // it on the channel every time
-                this.entityResolver = channel.getEntityResolver();
+            // listen to our channel events...
+            // note that we must reset listener on channel switch, as there is no
+            // guarantee that a new channel uses the same EventManager.
+            EventUtil.listenForChannelEvents(channel, mergeHandler);
+        }
 
-                EventManager eventManager = channel.getEventManager();
+        if (!usingSharedSnaphsotCache && getObjectStore() != null) {
+            DataRowStore cache = getObjectStore().getDataRowCache();
 
-                if (eventManager != null) {
-                    this.mergeHandler = new DataContextMergeHandler(this);
-
-                    // listen to our channel events...
-                    // note that we must reset listener on channel switch, as there is no
-                    // guarantee that a new channel uses the same EventManager.
-                    EventUtil.listenForChannelEvents(channel, mergeHandler);
-                }
-
-                if (!usingSharedSnaphsotCache && getObjectStore() != null) {
-                    DataRowStore cache = getObjectStore().getDataRowCache();
-
-                    if (cache != null) {
-                        cache.setEventManager(eventManager);
-                    }
-                }
+            if (cache != null) {
+                cache.setEventManager(eventManager);
             }
         }
     }
 
-    @Override
-    public DataChannel getChannel() {
-        awakeFromDeserialization();
-        return super.getChannel();
-    }
-
     /**
      * Returns a DataDomain used by this DataContext. DataDomain is looked up in the
      * DataChannel hierarchy. If a channel is not a DataDomain or a DataContext, null is
@@ -329,7 +180,7 @@ public class DataContext extends BaseContext implements DataChannel {
      * @since 1.1
      */
     public DataDomain getParentDataDomain() {
-        awakeFromDeserialization();
+        attachToRuntimeIfNeeded();
 
         if (channel == null) {
             return null;
@@ -476,6 +327,7 @@ public class DataContext extends BaseContext implements DataChannel {
         final ClassDescriptor descriptor = getEntityResolver().getClassDescriptor(
                 entity.getName());
         final DataRow snapshot = new DataRow(10);
+        snapshot.setEntityName(entity.getName());
 
         descriptor.visitProperties(new PropertyVisitor() {
 
@@ -483,8 +335,9 @@ public class DataContext extends BaseContext implements DataChannel {
                 ObjAttribute objAttr = property.getAttribute();
 
                 // processing compound attributes correctly
-                snapshot.put(objAttr.getDbAttributePath(), property
-                        .readPropertyDirectly(object));
+                snapshot.put(
+                        objAttr.getDbAttributePath(),
+                        property.readPropertyDirectly(object));
                 return true;
             }
 
@@ -565,26 +418,6 @@ public class DataContext extends BaseContext implements DataChannel {
         return snapshot;
     }
 
-    /**
-     * Converts a list of data rows to a list of DataObjects.
-     * 
-     * @since 1.1
-     * @deprecated since 3.0 as refreshing and resolvingInheritanceHierarchy flags are
-     *             deprecated. Use {@link #objectsFromDataRows(ClassDescriptor, List)}
-     *             instead.
-     */
-    @Deprecated
-    public List objectsFromDataRows(
-            ObjEntity entity,
-            List dataRows,
-            boolean refresh,
-            boolean resolveInheritanceHierarchy) {
-
-        ClassDescriptor descriptor = getEntityResolver().getClassDescriptor(
-                entity.getName());
-        return objectsFromDataRows(descriptor, dataRows);
-    }
-
     /**
      * Converts a list of DataRows to a List of DataObject registered with this
      * DataContext.
@@ -614,42 +447,28 @@ public class DataContext extends BaseContext implements DataChannel {
     }
 
     /**
-     * Converts a list of DataRows to a List of DataObject registered with this
-     * DataContext.
+     * Creates a DataObject from DataRow.
      * 
-     * @deprecated since 3.0 as refresh and resolveInheritanceHierarchy flags are
-     *             deprecated. Use {@link #objectsFromDataRows(ClassDescriptor, List)}
-     *             instead.
-     * @since 1.1
+     * @deprecated Use objectFromDataRow(Class<T> objectClass, DataRow dataRow) instead.
      * @see DataRow
      */
     @Deprecated
-    public List objectsFromDataRows(
-            Class<?> objectClass,
-            List<? extends DataRow> dataRows,
-            boolean refresh,
-            boolean resolveInheritanceHierarchy) {
-        ObjEntity entity = this.getEntityResolver().lookupObjEntity(objectClass);
-
-        if (entity == null) {
-            throw new CayenneRuntimeException("Unmapped Java class: " + objectClass);
-        }
-
-        ClassDescriptor descriptor = getEntityResolver().getClassDescriptor(
-                entity.getName());
-        return objectsFromDataRows(descriptor, dataRows);
+    public <T extends DataObject> T objectFromDataRow(
+            Class<T> objectClass,
+            DataRow dataRow,
+            boolean refresh) {
+        return objectFromDataRow(objectClass, dataRow);
     }
 
     /**
      * Creates a DataObject from DataRow.
      * 
      * @see DataRow
+     * @since 3.1
      */
     public <T extends DataObject> T objectFromDataRow(
             Class<T> objectClass,
-            DataRow dataRow,
-            boolean refresh) {
-
+            DataRow dataRow) {
         ObjEntity entity = this.getEntityResolver().lookupObjEntity(objectClass);
 
         if (entity == null) {
@@ -666,28 +485,32 @@ public class DataContext extends BaseContext implements DataChannel {
      * Creates a DataObject from DataRow. This variety of the 'objectFromDataRow' method
      * is normally used for generic classes.
      * 
+     * @deprecated Use objectFromDataRow(String entityName, DataRow dataRow) instead.
      * @see DataRow
      * @since 3.0
      */
+    @Deprecated
     public DataObject objectFromDataRow(
             String entityName,
             DataRow dataRow,
             boolean refresh) {
+        return objectFromDataRow(entityName, dataRow);
+    }
 
+    /**
+     * Creates a DataObject from DataRow. This variety of the 'objectFromDataRow' method
+     * is normally used for generic classes.
+     * 
+     * @see DataRow
+     * @since 3.1
+     */
+    public DataObject objectFromDataRow(String entityName, DataRow dataRow) {
         ClassDescriptor descriptor = getEntityResolver().getClassDescriptor(entityName);
         List<?> list = objectsFromDataRows(descriptor, Collections.singletonList(dataRow));
 
         return (DataObject) list.get(0);
     }
 
-    /**
-     * @deprecated since 3.0, use {@link #newObject(String)} instead.
-     */
-    @Deprecated
-    public DataObject createAndRegisterNewObject(String objEntityName) {
-        return (DataObject) newObject(objEntityName);
-    }
-
     /**
      * Creates and registers a new persistent object.
      * 
@@ -739,43 +562,12 @@ public class DataContext extends BaseContext implements DataChannel {
         // note that the order of initialization of persistence artifacts below is
         // important - do not change it lightly
         object.setObjectId(id);
-        object.setObjectContext(this);
-        object.setPersistenceState(PersistenceState.NEW);
-        getObjectStore().registerNode(id, object);
-        getObjectStore().nodeCreated(id);
-        
+
         injectInitialValue(object);
 
-        // invoke callbacks
-        getEntityResolver().getCallbackRegistry().performCallbacks(
-                LifecycleEvent.POST_ADD,
-                object);
-
         return object;
     }
 
-    /**
-     * Instantiates new object and registers it with itself. Object class must have a
-     * default constructor.
-     * 
-     * @since 1.1
-     * @deprecated since 3.0, use {@link #newObject(Class)} instead.
-     */
-    @Deprecated
-    public DataObject createAndRegisterNewObject(Class objectClass) {
-        if (objectClass == null) {
-            throw new NullPointerException("DataObject class can't be null.");
-        }
-
-        ObjEntity entity = getEntityResolver().lookupObjEntity(objectClass);
-        if (entity == null) {
-            throw new IllegalArgumentException("Class is not mapped with Cayenne: "
-                    + objectClass.getName());
-        }
-
-        return createAndRegisterNewObject(entity.getName());
-    }
-
     /**
      * Registers a transient object with the context, recursively registering all
      * transient persistent objects attached to this object via relationships.
@@ -817,21 +609,17 @@ public class DataContext extends BaseContext implements DataChannel {
             persistent.setObjectId(new ObjectId(entity.getName()));
         }
 
-        persistent.setObjectContext(this);
-        persistent.setPersistenceState(PersistenceState.NEW);
-
-        getObjectStore().registerNode(persistent.getObjectId(), object);
-        getObjectStore().nodeCreated(persistent.getObjectId());
-
-        // now we need to find all arc changes, inject missing value holders and pull in
-        // all transient connected objects
-
         ClassDescriptor descriptor = getEntityResolver().getClassDescriptor(
                 entity.getName());
         if (descriptor == null) {
             throw new IllegalArgumentException("Invalid entity name: " + entity.getName());
         }
 
+        injectInitialValue(object);
+
+        // now we need to find all arc changes, inject missing value holders and pull in
+        // all transient connected objects
+
         descriptor.visitProperties(new PropertyVisitor() {
 
             public boolean visitToMany(ToManyProperty property) {
@@ -884,13 +672,6 @@ public class DataContext extends BaseContext implements DataChannel {
                 return true;
             }
         });
-        
-        injectInitialValue(object);
-
-        // invoke callbacks
-        getEntityResolver().getCallbackRegistry().performCallbacks(
-                LifecycleEvent.POST_ADD,
-                persistent);
     }
 
     /**
@@ -904,91 +685,6 @@ public class DataContext extends BaseContext implements DataChannel {
         getObjectStore().objectsUnregistered(dataObjects);
     }
 
-    /**
-     * Schedules all objects in the collection for deletion on the next commit of this
-     * DataContext. Object's persistence state is changed to PersistenceState.DELETED;
-     * objects related to this object are processed according to delete rules, i.e.
-     * relationships can be unset ("nullify" rule), deletion operation is cascaded
-     * (cascade rule).
-     * <p>
-     * <i>"Nullify" delete rule side effect: </i> passing a collection representing
-     * to-many relationship with nullify delete rule may result in objects being removed
-     * from collection.
-     * </p>
-     * 
-     * @since 1.2
-     */
-    public void deleteObjects(Collection objects) {
-        if (objects.isEmpty()) {
-            return;
-        }
-
-        // clone object list... this maybe a relationship collection with nullify delete
-        // rule, so modifying
-        for (Persistent object : new ArrayList<Persistent>(objects)) {
-            deleteObject(object);
-        }
-    }
-
-    /**
-     * Schedules an object for deletion on the next commit of this DataContext. Object's
-     * persistence state is changed to PersistenceState.DELETED; objects related to this
-     * object are processed according to delete rules, i.e. relationships can be unset
-     * ("nullify" rule), deletion operation is cascaded (cascade rule).
-     * 
-     * @param object a persistent object that we want to delete.
-     * @throws DeleteDenyException if a DENY delete rule is applicable for object
-     *             deletion.
-     * @throws NullPointerException if object is null.
-     */
-    @Override
-    public void deleteObject(Object object) throws DeleteDenyException {
-        new DataContextDeleteAction(this).performDelete((Persistent) object);
-    }
-
-    /**
-     * Refetches object data for ObjectId. This method is used internally by Cayenne to
-     * resolve objects in state <code>PersistenceState.HOLLOW</code>. It can also be used
-     * to refresh certain objects.
-     * 
-     * @throws CayenneRuntimeException if object id doesn't match any records, or if there
-     *             is more than one object is fetched.
-     * @deprecated since 3.0 use {@link ObjectIdQuery} with appropriate refresh settings.
-     */
-    @Deprecated
-    public DataObject refetchObject(ObjectId oid) {
-
-        if (oid == null) {
-            throw new NullPointerException("Null ObjectId");
-        }
-
-        if (oid.isTemporary()) {
-            throw new CayenneRuntimeException("Can't refetch ObjectId "
-                    + oid
-                    + ", as it is a temporary id.");
-        }
-
-        synchronized (getObjectStore()) {
-            DataObject object = (DataObject) objectStore.getNode(oid);
-
-            // clean up any cached data for this object
-            if (object != null) {
-                this.invalidateObjects(Collections.singleton(object));
-            }
-        }
-
-        DataObject object = (DataObject) DataObjectUtils.objectForQuery(
-                this,
-                new ObjectIdQuery(oid));
-
-        if (object == null) {
-            throw new CayenneRuntimeException(
-                    "Refetch failure: no matching objects found for ObjectId " + oid);
-        }
-
-        return object;
-    }
-
     /**
      * If the parent channel is a DataContext, reverts local changes to make this context
      * look like the parent, if the parent channel is a DataDomain, reverts all changes.
@@ -1362,16 +1058,6 @@ public class DataContext extends BaseContext implements DataChannel {
         return performQuery(query);
     }
 
-    /**
-     * Returns EntityResolver. EntityResolver can be null if DataContext has not been
-     * attached to an DataChannel.
-     */
-    @Override
-    public EntityResolver getEntityResolver() {
-        awakeFromDeserialization();
-        return entityResolver;
-    }
-
     /**
      * Returns <code>true</code> if the ObjectStore uses shared cache of a parent
      * DataDomain.
@@ -1383,23 +1069,10 @@ public class DataContext extends BaseContext implements DataChannel {
     }
 
     /**
-     * Returns whether this DataContext performs object validation before commit is
-     * executed.
-     * 
-     * @since 1.1
+     * @since 3.1
      */
-    public boolean isValidatingObjectsOnCommit() {
-        return validatingObjectsOnCommit;
-    }
-
-    /**
-     * Sets the property defining whether this DataContext should perform object
-     * validation before commit is executed.
-     * 
-     * @since 1.1
-     */
-    public void setValidatingObjectsOnCommit(boolean flag) {
-        this.validatingObjectsOnCommit = flag;
+    public void setUsingSharedSnapshotCache(boolean flag) {
+        this.usingSharedSnaphsotCache = flag;
     }
 
     // ---------------------------------------------
@@ -1409,23 +1082,6 @@ public class DataContext extends BaseContext implements DataChannel {
     private void writeObject(ObjectOutputStream out) throws IOException {
         out.defaultWriteObject();
 
-        // If the "parent" of this datacontext is a DataDomain, then just write the
-        // name of it. Then when deserialization happens, we can get back the DataDomain
-        // by name, from the shared configuration (which will either load it if need be,
-        // or return an existing one.
-
-        if (this.channel == null && this.lazyInitParentDomainName != null) {
-            out.writeObject(lazyInitParentDomainName);
-        }
-        else if (this.channel instanceof DataDomain) {
-            DataDomain domain = (DataDomain) this.channel;
-            out.writeObject(domain.getName());
-        }
-        else {
-            // Hope that whatever this.parent is, that it is Serializable
-            out.writeObject(this.channel);
-        }
-
         // Serialize local snapshots cache
         if (!isUsingSharedSnapshotCache()) {
             out.writeObject(objectStore.getDataRowCache());
@@ -1436,74 +1092,36 @@ public class DataContext extends BaseContext implements DataChannel {
     private void readObject(ObjectInputStream in) throws IOException,
             ClassNotFoundException {
 
-        // 1. read non-transient properties
+        // TODO: most of this should be in the superclass, especially the code connecting
+        // super transient ivars
+
+        // read non-transient properties
         in.defaultReadObject();
 
-        // 2. read parent or its name
-        Object value = in.readObject();
-        if (value instanceof DataChannel) {
-            // A real QueryEngine object - use it
-        	// call a setter to initialize EntityResolver 
-            setChannel((DataChannel) value);
-        }
-        else if (value instanceof String) {
-            // The name of a DataDomain - use it
-            this.lazyInitParentDomainName = (String) value;
-        }
-        else {
-            throw new CayenneRuntimeException(
-                    "Parent attribute of DataContext was neither a QueryEngine nor "
-                            + "the name of a valid DataDomain:"
-                            + value);
-        }
-
-        // 3. Deserialize local snapshots cache
+        // deserialize local snapshots cache
         if (!isUsingSharedSnapshotCache()) {
             DataRowStore cache = (DataRowStore) in.readObject();
             objectStore.setDataRowCache(cache);
         }
 
-        // CayenneDataObjects have a transient datacontext
+        // CayenneDataObjects have a transient DataContext
         // because at deserialize time the datacontext may need to be different
         // than the one at serialize time (for programmer defined reasons).
-        // So, when a dataobject is resurrected because it's datacontext was
-        // serialized, it will then set the objects datacontext to the correctone
-        // If deserialized "otherwise", it will not have a datacontext (good)
+        // So, when a DataObject is resurrected because it's DataContext was
+        // serialized, it will then set the objects DataContext to the correct one
+        // If deserialized "otherwise", it will not have a DataContext.
 
         synchronized (getObjectStore()) {
-            Iterator it = objectStore.getObjectIterator();
+            Iterator<?> it = objectStore.getObjectIterator();
             while (it.hasNext()) {
                 Persistent object = (Persistent) it.next();
                 object.setObjectContext(this);
             }
         }
-    }
 
-    // Re-attaches itself to the parent domain with previously stored name.
-    //
-    // TODO: Andrus 11/7/2005 - this is one of the places where Cayenne
-    // serialization relies on shared config... This is bad. We need some
-    // sort of thread-local solution that would allow to use an alternative configuration.
-    //
-    private final void awakeFromDeserialization() {
-        if (channel == null && lazyInitParentDomainName != null) {
-
-            // call a setter to ensure EntityResolver is extracted from channel
-            setChannel(Configuration.getSharedConfiguration().getDomain(
-                    lazyInitParentDomainName));
-        }
-    }
-
-    /**
-     * @since 1.2
-     */
-    @Override
-    public void propertyChanged(
-            Persistent object,
-            String property,
-            Object oldValue,
-            Object newValue) {
-        graphAction.handlePropertyChange(object, property, oldValue, newValue);
+        // ... deferring initialization of transient properties of this context till first
+        // access, so that it can attach to Cayenne runtime using appropriate thread
+        // injector.
     }
 
     /**
@@ -1517,32 +1135,18 @@ public class DataContext extends BaseContext implements DataChannel {
     }
 
     /**
-     * Returns an object local to this DataContext and matching the ObjectId. If
-     * <code>prototype</code> is not null, local object is refreshed with the prototype
-     * values.
-     * <p>
-     * In case you pass a non-null second parameter, you are responsible for setting
-     * correct persistence state of the returned local object, as generally there is no
-     * way for Cayenne to determine the resulting local object state.
+     * An internal version of {@link #localObject(Object)} that operates on ObjectId
+     * instead of Persistent, and wouldn't attempt to look up an object in the parent
+     * channel.
      * 
-     * @since 1.2
+     * @since 3.1
      */
-    @Override
-    public Persistent localObject(ObjectId id, Object prototype) {
-
-        // ****** Warning: when changing the code below, don't forget to change
-        // CayenneContext's implementation which right now relies on copy/paste "reuse"
+    Persistent findOrCreateObject(ObjectId id) {
 
         if (id == null) {
             throw new IllegalArgumentException("Null ObjectId");
         }
 
-        // note that per-object ClassDescriptor lookup is needed as even if all
-        // objects where fetched as a part of the same query, as they may belong to
-        // different subclasses
-        ClassDescriptor descriptor = getEntityResolver().getClassDescriptor(
-                id.getEntityName());
-
         // have to synchronize almost the entire method to prevent multiple threads from
         // messing up dataobjects per CAY-845. Originally only parts of "else" were
         // synchronized, but we had to expand the lock scope to ensure consistent
@@ -1550,58 +1154,43 @@ public class DataContext extends BaseContext implements DataChannel {
         synchronized (getGraphManager()) {
             Persistent cachedObject = (Persistent) getGraphManager().getNode(id);
 
-            // merge into an existing object
+            // return an existing object
             if (cachedObject != null) {
 
                 int state = cachedObject.getPersistenceState();
 
                 // TODO: Andrus, 1/24/2006 implement smart merge for modified objects...
-                if (cachedObject != prototype
-                        && state != PersistenceState.MODIFIED
+                if (state != PersistenceState.MODIFIED
                         && state != PersistenceState.DELETED) {
 
+                    ClassDescriptor descriptor = getEntityResolver().getClassDescriptor(
+                            id.getEntityName());
+
                     descriptor.injectValueHolders(cachedObject);
-
-                    if (prototype != null
-                            && ((Persistent) prototype).getPersistenceState() != PersistenceState.HOLLOW) {
-
-                        descriptor.shallowMerge(prototype, cachedObject);
-
-                        if (state == PersistenceState.HOLLOW) {
-                            cachedObject.setPersistenceState(PersistenceState.COMMITTED);
-                        }
-                    }
                 }
 
                 return cachedObject;
             }
-            // create and merge into a new object
-            else {
 
-                Persistent localObject;
+            // create and register a hollow object
+            ClassDescriptor descriptor = getEntityResolver().getClassDescriptor(
+                    id.getEntityName());
+            Persistent localObject = (Persistent) descriptor.createObject();
 
-                localObject = (Persistent) descriptor.createObject();
+            localObject.setObjectContext(this);
+            localObject.setObjectId(id);
 
-                localObject.setObjectContext(this);
-                localObject.setObjectId(id);
+            getGraphManager().registerNode(id, localObject);
+            localObject.setPersistenceState(PersistenceState.HOLLOW);
 
-                getGraphManager().registerNode(id, localObject);
-
-                if (prototype != null
-                        && ((Persistent) prototype).getPersistenceState() != PersistenceState.HOLLOW) {
-                    localObject.setPersistenceState(PersistenceState.COMMITTED);
-                    descriptor.injectValueHolders(localObject);
-                    descriptor.shallowMerge(prototype, localObject);
-                }
-                else {
-                    localObject.setPersistenceState(PersistenceState.HOLLOW);
-                }
-
-                return localObject;
-            }
+            return localObject;
         }
+
     }
 
+    // this completely meaningless override is needed to expose the method as
+    // package-private ... is there a better way?
+    @Override
     protected void fireDataChannelChanged(Object postedBy, GraphDiff changes) {
         super.fireDataChannelChanged(postedBy, changes);
     }
