@@ -40,7 +40,9 @@ import org.apache.cayenne.QueryResponse;
 import org.apache.cayenne.cache.QueryCache;
 import org.apache.cayenne.cache.QueryCacheEntryFactory;
 import org.apache.cayenne.map.DataMap;
+import org.apache.cayenne.map.DbEntity;
 import org.apache.cayenne.map.DbRelationship;
+import org.apache.cayenne.map.EntityInheritanceTree;
 import org.apache.cayenne.map.LifecycleEvent;
 import org.apache.cayenne.map.ObjRelationship;
 import org.apache.cayenne.query.EntityResultSegment;
@@ -145,11 +147,21 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
         if (query instanceof ObjectIdQuery) {
 
             ObjectIdQuery oidQuery = (ObjectIdQuery) query;
+            ObjectId oid = oidQuery.getObjectId();
+
+            // special handling of temp ids... Return an empty list immediately so that
+            // upstream code could throw FaultFailureException, etc. Don't attempt to
+            // translate and run the query. See for instance CAY-1651
+            if (oid.isTemporary() && !oid.isReplacementIdAttached()) {
+                response = new ListResponse();
+                return DONE;
+            }
+
             DataRow row = null;
 
-            if (cache != null && !oidQuery.isFetchMandatory()) {
-                row = cache.getCachedSnapshot(oidQuery.getObjectId());
-            }
+			if (cache != null && !oidQuery.isFetchMandatory()) {
+				row = polymorphicRowFromCache(oid);
+			}
 
             // refresh is forced or not found in cache
             if (row == null) {
@@ -171,6 +183,38 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
 
         return !DONE;
     }
+    
+	private DataRow polymorphicRowFromCache(ObjectId superOid) {
+		DataRow row = cache.getCachedSnapshot(superOid);
+		if (row != null) {
+			return row;
+		}
+
+		EntityInheritanceTree inheritanceTree = domain.getEntityResolver().lookupInheritanceTree(superOid.getEntityName());
+		if (!inheritanceTree.getChildren().isEmpty()) {
+			row = polymorphicRowFromCache(inheritanceTree, superOid.getIdSnapshot());
+		}
+
+		return row;
+	}
+    
+	private DataRow polymorphicRowFromCache(EntityInheritanceTree superNode, Map<String, ?> idSnapshot) {
+
+		for (EntityInheritanceTree child : superNode.getChildren()) {
+			ObjectId id = new ObjectId(child.getEntity().getName(), idSnapshot);
+			DataRow row = cache.getCachedSnapshot(id);
+			if (row != null) {
+				return row;
+			}
+			
+			row = polymorphicRowFromCache(child, idSnapshot);
+			if (row != null) {
+				return row;
+			}
+		}
+
+		return null;
+	}
 
     private boolean interceptRelationshipQuery() {
 
@@ -184,11 +228,23 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
             ObjRelationship relationship = relationshipQuery.getRelationship(domain
                     .getEntityResolver());
 
-            // check if we can derive target PK from FK... this implies that the
-            // relationship is to-one
+            // check if we can derive target PK from FK...
             if (relationship.isSourceIndependentFromTargetChange()) {
                 return !DONE;
             }
+            
+            // we can assume that there is one and only one DbRelationship as
+            // we previously checked that "!isSourceIndependentFromTargetChange"
+            DbRelationship dbRelationship = relationship.getDbRelationships().get(0);
+            
+            // FK pointing to a unique field that is a 'fake' PK (CAY-1755)...
+            // It is not sufficient to generate target ObjectId.
+            DbEntity targetEntity = (DbEntity) dbRelationship.getTargetEntity();
+            if (dbRelationship.getJoins().size() < targetEntity
+                    .getPrimaryKeys().size()) {
+                return !DONE;
+            }
+            
 
             if (cache == null) {
                 return !DONE;
@@ -199,12 +255,10 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
                 return !DONE;
             }
 
-            // we can assume that there is one and only one DbRelationship as
-            // we previously checked that "!isSourceIndependentFromTargetChange"
-            DbRelationship dbRelationship = relationship.getDbRelationships().get(0);
 
-            ObjectId targetId = sourceRow.createTargetObjectId(relationship
-                    .getTargetEntityName(), dbRelationship);
+            ObjectId targetId = sourceRow.createTargetObjectId(
+                    relationship.getTargetEntityName(),
+                    dbRelationship);
 
             // null id means that FK is null...
             if (targetId == null) {
@@ -212,7 +266,8 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
                 return DONE;
             }
 
-            DataRow targetRow = cache.getCachedSnapshot(targetId);
+            // target id resolution (unlike source) should be polymorphic
+            DataRow targetRow = polymorphicRowFromCache(targetId);
 
             if (targetRow != null) {
                 this.response = new GenericResponse(Collections.singletonList(targetRow));
@@ -226,10 +281,9 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
                             .getEntityResolver())) {
 
                 // prevent passing partial snapshots to ObjectResolver per CAY-724.
-                // Create
-                // a hollow object right here and skip object conversion downstream
+                // Create a hollow object right here and skip object conversion downstream
                 this.noObjectConversion = true;
-                Object object = context.localObject(targetId, null);
+                Object object = context.findOrCreateObject(targetId);
 
                 this.response = new GenericResponse(Collections.singletonList(object));
                 return DONE;
@@ -252,6 +306,9 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
                 // not sending any events - peer contexts will not get refreshed
                 if (domain.getSharedSnapshotCache() != null) {
                     domain.getSharedSnapshotCache().clear();
+                } else {
+                    // remove snapshots from local ObjectStore only
+                    context.getObjectStore().getDataRowCache().clear();
                 }
                 context.getQueryCache().clear();
 
@@ -278,6 +335,14 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
                             Collections.EMPTY_LIST,
                             ids,
                             Collections.EMPTY_LIST);
+                } else {
+                    // remove snapshots from local ObjectStore only
+                    context.getObjectStore().getDataRowCache().processSnapshotChanges(
+                            context.getObjectStore(),
+                            Collections.EMPTY_MAP,
+                            Collections.EMPTY_LIST,
+                            ids,
+                            Collections.EMPTY_LIST);
                 }
 
                 GenericResponse response = new GenericResponse();
@@ -568,8 +633,10 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
 
             // take a shortcut when no prefetches exist...
             if (prefetchTree == null) {
-                return new ObjectResolver(context, descriptor, metadata
-                        .isRefreshingObjects())
+                return new ObjectResolver(
+                        context,
+                        descriptor,
+                        metadata.isRefreshingObjects())
                         .synchronizedRootResultNodeFromDataRows(normalizedRows);
             }
             else {
@@ -678,8 +745,10 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
             }
 
             if (prefetchTree == null) {
-                return new ObjectResolver(context, descriptor, metadata
-                        .isRefreshingObjects())
+                return new ObjectResolver(
+                        context,
+                        descriptor,
+                        metadata.isRefreshingObjects())
                         .synchronizedRootResultNodeFromDataRows(rowsColumn);
             }
             else {
@@ -705,7 +774,8 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
 
             // no conversions needed for scalar positions; reuse Object[]'s to fill them
             // with resolved objects
-            List<PrefetchProcessorNode> segmentNodes = new ArrayList<PrefetchProcessorNode>(width);
+            List<PrefetchProcessorNode> segmentNodes = new ArrayList<PrefetchProcessorNode>(
+                    width);
             for (int i = 0; i < width; i++) {
 
                 if (rsMapping.get(i) instanceof EntityResultSegment) {
@@ -718,7 +788,7 @@ class DataDomainQueryAction implements QueryRouter, OperationObserver {
                             i);
 
                     segmentNodes.add(nextResult);
-                    
+
                     List<Persistent> objects = nextResult.getObjects();
 
                     for (int j = 0; j < rowsLen; j++) {
