@@ -28,6 +28,7 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.concurrent.ConcurrentMap;
 
 import org.apache.cayenne.CayenneRuntimeException;
 import org.apache.cayenne.DataObject;
@@ -40,17 +41,13 @@ import org.apache.cayenne.event.EventBridge;
 import org.apache.cayenne.event.EventBridgeFactory;
 import org.apache.cayenne.event.EventManager;
 import org.apache.cayenne.event.EventSubject;
+import org.apache.cayenne.util.concurrentlinkedhashmap.ConcurrentLinkedHashMap;
 import org.apache.commons.collections.ExtendedProperties;
-import org.apache.commons.collections.map.LRUMap;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
 /**
  * A fixed size cache of DataRows keyed by ObjectId.
- * <p>
- * <strong>Synchronization Note: </strong> DataRowStore synchronizes most operations on
- * its own instance.
- * </p>
  * 
  * @since 1.1
  */
@@ -76,7 +73,8 @@ public class DataRowStore implements Serializable {
     public static final String EVENT_BRIDGE_FACTORY_DEFAULT = "org.apache.cayenne.event.JavaGroupsBridgeFactory";
 
     protected String name;
-    protected LRUMap snapshots;
+    private int maxSize;
+    protected ConcurrentMap<ObjectId, DataRow> snapshots;
     protected boolean notifyingRemoteListeners;
 
     protected transient EventManager eventManager;
@@ -86,31 +84,6 @@ public class DataRowStore implements Serializable {
     // too early, and thus disabling events.
     protected transient EventSubject eventSubject;
 
-    /**
-     * Creates new named DataRowStore with default configuration.
-     * 
-     * @deprecated since 3.0 as it instantiates an EventManager internally that can lead
-     *             to dispatch thread leaks.
-     */
-    public DataRowStore(String name) {
-        this(name, Collections.EMPTY_MAP);
-    }
-
-    /**
-     * Creates new DataRowStore with a specified name and a set of properties. If no
-     * properties are defined, default values are used.
-     * 
-     * @param name DataRowStore name. Used to idenitfy this DataRowStore in events, etc.
-     *            Can't be null.
-     * @param properties Properties map used to configure DataRowStore parameters. Can be
-     *            null.
-     * @deprecated since 3.0 as it instantiates an EventManager internally that can lead
-     *             to dispatch thread leaks.
-     */
-    public DataRowStore(String name, Map properties) {
-        this(name, properties, new EventManager());
-    }
-
     /**
      * Creates new DataRowStore with a specified name and a set of properties. If no
      * properties are defined, default values are used.
@@ -149,7 +122,7 @@ public class DataRowStore implements Serializable {
                 SNAPSHOT_EXPIRATION_PROPERTY,
                 SNAPSHOT_EXPIRATION_DEFAULT);
 
-        int snapshotsCacheSize = propertiesWrapper.getInt(
+        maxSize = propertiesWrapper.getInt(
                 SNAPSHOT_CACHE_SIZE_PROPERTY,
                 SNAPSHOT_CACHE_SIZE_DEFAULT);
 
@@ -169,7 +142,7 @@ public class DataRowStore implements Serializable {
             logger.debug("DataRowStore property "
                     + SNAPSHOT_CACHE_SIZE_PROPERTY
                     + " = "
-                    + snapshotsCacheSize);
+                    + maxSize);
             logger.debug("DataRowStore property "
                     + REMOTE_NOTIFICATION_PROPERTY
                     + " = "
@@ -183,8 +156,9 @@ public class DataRowStore implements Serializable {
         // init ivars from properties
         this.notifyingRemoteListeners = notifyRemote;
 
-        // TODO: ENTRY EXPIRATION is not supported by commons LRU Map
-        this.snapshots = new LRUMap(snapshotsCacheSize);
+        this.snapshots = new ConcurrentLinkedHashMap.Builder<ObjectId, DataRow>()
+                .maximumWeightedCapacity(maxSize)
+                .build();
 
         // init event bridge only if we are notifying remote listeners
         if (notifyingRemoteListeners) {
@@ -192,7 +166,8 @@ public class DataRowStore implements Serializable {
                 EventBridgeFactory factory = (EventBridgeFactory) Class.forName(
                         eventBridgeFactory).newInstance();
 
-                Collection<EventSubject> subjects = Collections.singleton(getSnapshotEventSubject());
+                Collection<EventSubject> subjects = Collections
+                        .singleton(getSnapshotEventSubject());
                 String externalSubject = EventBridge
                         .convertToExternalSubject(getSnapshotEventSubject());
                 this.remoteNotificationsHandler = factory.createEventBridge(
@@ -230,60 +205,58 @@ public class DataRowStore implements Serializable {
         Map modified = null;
         Object eventPostedBy = null;
 
-        synchronized (this) {
-            for (int i = 0; i < size; i++) {
-                Persistent object = (Persistent) objects.get(i);
-                
-                // skip null objects... possible since 3.0 in some EJBQL results
-                if (object == null) {
-                    continue;
-                }
+        for (int i = 0; i < size; i++) {
+            Persistent object = (Persistent) objects.get(i);
 
-                // skip HOLLOW objects as they likely were created from partial snapshots
-                if (object.getPersistenceState() == PersistenceState.HOLLOW) {
-                    continue;
-                }
-
-                ObjectId oid = object.getObjectId();
-
-                // add snapshots if refresh is forced, or if a snapshot is
-                // missing
-
-                DataRow cachedSnapshot = (DataRow) this.snapshots.get(oid);
-                if (refresh || cachedSnapshot == null) {
-
-                    DataRow newSnapshot = (DataRow) snapshots.get(i);
-
-                    if (cachedSnapshot != null) {
-                        // use old snapshot if no changes occurred
-                        if (object instanceof DataObject
-                                && cachedSnapshot.equals(newSnapshot)) {
-                            ((DataObject) object).setSnapshotVersion(cachedSnapshot
-                                    .getVersion());
-                            continue;
-                        }
-                        else {
-                            newSnapshot.setReplacesVersion(cachedSnapshot.getVersion());
-                        }
-                    }
-
-                    if (modified == null) {
-                        modified = new HashMap();
-                        eventPostedBy = object.getObjectContext().getGraphManager();
-                    }
-
-                    modified.put(oid, newSnapshot);
-                }
+            // skip null objects... possible since 3.0 in some EJBQL results
+            if (object == null) {
+                continue;
             }
 
-            if (modified != null) {
-                processSnapshotChanges(
-                        eventPostedBy,
-                        modified,
-                        Collections.EMPTY_LIST,
-                        Collections.EMPTY_LIST,
-                        Collections.EMPTY_LIST);
+            // skip HOLLOW objects as they likely were created from partial snapshots
+            if (object.getPersistenceState() == PersistenceState.HOLLOW) {
+                continue;
             }
+
+            ObjectId oid = object.getObjectId();
+
+            // add snapshots if refresh is forced, or if a snapshot is
+            // missing
+
+            DataRow cachedSnapshot = this.snapshots.get(oid);
+            if (refresh || cachedSnapshot == null) {
+
+                DataRow newSnapshot = (DataRow) snapshots.get(i);
+
+                if (cachedSnapshot != null) {
+                    // use old snapshot if no changes occurred
+                    if (object instanceof DataObject
+                            && cachedSnapshot.equals(newSnapshot)) {
+                        ((DataObject) object).setSnapshotVersion(cachedSnapshot
+                                .getVersion());
+                        continue;
+                    }
+                    else {
+                        newSnapshot.setReplacesVersion(cachedSnapshot.getVersion());
+                    }
+                }
+
+                if (modified == null) {
+                    modified = new HashMap();
+                    eventPostedBy = object.getObjectContext().getGraphManager();
+                }
+
+                modified.put(oid, newSnapshot);
+            }
+        }
+
+        if (modified != null) {
+            processSnapshotChanges(
+                    eventPostedBy,
+                    modified,
+                    Collections.EMPTY_LIST,
+                    Collections.EMPTY_LIST,
+                    Collections.EMPTY_LIST);
         }
     }
 
@@ -298,7 +271,7 @@ public class DataRowStore implements Serializable {
      * Returns maximum allowed cache size.
      */
     public int maximumSize() {
-        return snapshots.maxSize();
+        return maxSize;
     }
 
     /**
@@ -351,8 +324,8 @@ public class DataRowStore implements Serializable {
      * Returns cached snapshot or null if no snapshot is currently cached for the given
      * ObjectId.
      */
-    public synchronized DataRow getCachedSnapshot(ObjectId oid) {
-        return (DataRow) snapshots.get(oid);
+    public DataRow getCachedSnapshot(ObjectId oid) {
+        return snapshots.get(oid);
     }
 
     /**
@@ -365,14 +338,14 @@ public class DataRowStore implements Serializable {
     /**
      * Expires and removes all stored snapshots without sending any notification events.
      */
-    public synchronized void clear() {
+    public void clear() {
         snapshots.clear();
     }
 
     /**
      * Evicts a snapshot from cache without generating any SnapshotEvents.
      */
-    public synchronized void forgetSnapshot(ObjectId id) {
+    public void forgetSnapshot(ObjectId id) {
         snapshots.remove(id);
     }
 
@@ -402,17 +375,15 @@ public class DataRowStore implements Serializable {
             return;
         }
 
-        synchronized (this) {
-            processDeletedIDs(deletedSnapshotIds);
-            processInvalidatedIDs(invalidatedSnapshotIds);
-            processUpdateDiffs(diffs);
-            sendUpdateNotification(
-                    event.getPostedBy(),
-                    diffs,
-                    deletedSnapshotIds,
-                    invalidatedSnapshotIds,
-                    indirectlyModifiedIds);
-        }
+        processDeletedIDs(deletedSnapshotIds);
+        processInvalidatedIDs(invalidatedSnapshotIds);
+        processUpdateDiffs(diffs);
+        sendUpdateNotification(
+                event.getPostedBy(),
+                diffs,
+                deletedSnapshotIds,
+                invalidatedSnapshotIds,
+                indirectlyModifiedIds);
     }
 
     /**
@@ -436,17 +407,15 @@ public class DataRowStore implements Serializable {
             return;
         }
 
-        synchronized (this) {
-            processDeletedIDs(deletedSnapshotIds);
-            processInvalidatedIDs(invalidatedSnapshotIds);
-            Map diffs = processUpdatedSnapshots(updatedSnapshots);
-            sendUpdateNotification(
-                    postedBy,
-                    diffs,
-                    deletedSnapshotIds,
-                    invalidatedSnapshotIds,
-                    indirectlyModifiedIds);
-        }
+        processDeletedIDs(deletedSnapshotIds);
+        processInvalidatedIDs(invalidatedSnapshotIds);
+        Map diffs = processUpdatedSnapshots(updatedSnapshots);
+        sendUpdateNotification(
+                postedBy,
+                diffs,
+                deletedSnapshotIds,
+                invalidatedSnapshotIds,
+                indirectlyModifiedIds);
     }
 
     private void processDeletedIDs(Collection deletedSnapshotIDs) {
@@ -480,7 +449,7 @@ public class DataRowStore implements Serializable {
 
                 ObjectId key = (ObjectId) entry.getKey();
                 DataRow newSnapshot = (DataRow) entry.getValue();
-                DataRow oldSnapshot = (DataRow) snapshots.put(key, newSnapshot);
+                DataRow oldSnapshot = snapshots.put(key, newSnapshot);
 
                 // generate diff for the updated event, if this not a new
                 // snapshot
@@ -504,11 +473,17 @@ public class DataRowStore implements Serializable {
                     // case 4 above... have to throw out the snapshot since
                     // no good options exist to tell how to merge the two.
                     if (oldSnapshot.getVersion() != newSnapshot.getReplacesVersion()) {
-                        logger
-                                .debug("snapshot version changed, don't know what to do... Old: "
-                                        + oldSnapshot
-                                        + ", New: "
-                                        + newSnapshot);
+
+                        // snapshots can be huge potentially.. so print them only if the
+                        // user is expecting them to be printed
+                        if (logger.isDebugEnabled()) {
+                            logger
+                                    .debug("snapshot version changed, don't know what to do... Old: "
+                                            + oldSnapshot
+                                            + ", New: "
+                                            + newSnapshot);
+                        }
+
                         forgetSnapshot(key);
                         continue;
                     }
@@ -536,7 +511,7 @@ public class DataRowStore implements Serializable {
             while (it.hasNext()) {
                 Map.Entry entry = (Map.Entry) it.next();
                 ObjectId key = (ObjectId) entry.getKey();
-                DataRow oldSnapshot = (DataRow) snapshots.remove(key);
+                DataRow oldSnapshot = snapshots.remove(key);
 
                 if (oldSnapshot == null) {
                     continue;
