@@ -34,16 +34,21 @@ import java.util.Map;
 
 import javax.sql.DataSource;
 
+import org.apache.cayenne.ashwood.AshwoodEntitySorter;
 import org.apache.cayenne.conn.DataSourceInfo;
 import org.apache.cayenne.conn.DriverDataSource;
 import org.apache.cayenne.dba.DbAdapter;
 import org.apache.cayenne.dba.PkGenerator;
 import org.apache.cayenne.dba.TypesMapping;
+import org.apache.cayenne.log.JdbcEventLogger;
+import org.apache.cayenne.log.NoopJdbcEventLogger;
 import org.apache.cayenne.map.DataMap;
 import org.apache.cayenne.map.DbAttribute;
 import org.apache.cayenne.map.DbEntity;
 import org.apache.cayenne.map.DbJoin;
 import org.apache.cayenne.map.DbRelationship;
+import org.apache.cayenne.map.EntityResolver;
+import org.apache.cayenne.map.EntitySorter;
 import org.apache.cayenne.validation.SimpleValidationFailure;
 import org.apache.cayenne.validation.ValidationResult;
 import org.apache.commons.logging.Log;
@@ -52,7 +57,6 @@ import org.apache.commons.logging.LogFactory;
 /**
  * Utility class that generates database schema based on Cayenne mapping. It is a logical
  * counterpart of DbLoader class.
- * 
  */
 public class DbGenerator {
 
@@ -64,6 +68,8 @@ public class DbGenerator {
     // optional DataDomain needed for correct FK generation in cross-db situations
     protected DataDomain domain;
 
+    protected JdbcEventLogger jdbcEventLogger;
+
     // stores generated SQL statements
     protected Map<String, Collection<String>> dropTables;
     protected Map<String, String> createTables;
@@ -88,7 +94,10 @@ public class DbGenerator {
 
     /**
      * Creates and initializes new DbGenerator.
+     * 
+     * @deprecated since 3.1 use {@link #DbGenerator(DbAdapter, DataMap, JdbcEventLogger)}
      */
+    @Deprecated
     public DbGenerator(DbAdapter adapter, DataMap map) {
         this(adapter, map, Collections.<DbEntity> emptyList());
     }
@@ -99,10 +108,28 @@ public class DbGenerator {
      * @param adapter DbAdapter corresponding to the database
      * @param map DataMap whose entities will be used in schema generation
      * @param excludedEntities entities that should be ignored during schema generation
+     * @deprecated since 3.1 use
+     *             {@link #DbGenerator(DbAdapter, DataMap, Collection, DataDomain, JdbcEventLogger)}
      */
+    @Deprecated
     public DbGenerator(DbAdapter adapter, DataMap map,
             Collection<DbEntity> excludedEntities) {
-        this(adapter, map, excludedEntities, null);
+        this(adapter, map, excludedEntities, null, NoopJdbcEventLogger.getInstance());
+    }
+
+    /**
+     * @since 3.1
+     */
+    public DbGenerator(DbAdapter adapter, DataMap map, JdbcEventLogger logger) {
+        this(adapter, map, logger, Collections.<DbEntity> emptyList());
+    }
+
+    /**
+     * @since 3.1
+     */
+    public DbGenerator(DbAdapter adapter, DataMap map, JdbcEventLogger logger,
+            Collection<DbEntity> excludedEntities) {
+        this(adapter, map, excludedEntities, null, logger);
     }
 
     /**
@@ -112,10 +139,11 @@ public class DbGenerator {
      * @param map DataMap whose entities will be used in schema generation
      * @param excludedEntities entities that should be ignored during schema generation
      * @param domain optional DataDomain used to detect cross-database relationships.
-     * @since 1.2
+     * @since 3.1
      */
     public DbGenerator(DbAdapter adapter, DataMap map,
-            Collection<DbEntity> excludedEntities, DataDomain domain) {
+            Collection<DbEntity> excludedEntities, DataDomain domain,
+            JdbcEventLogger logger) {
         // sanity check
         if (adapter == null) {
             throw new IllegalArgumentException("Adapter must not be null.");
@@ -128,6 +156,7 @@ public class DbGenerator {
         this.domain = domain;
         this.map = map;
         this.adapter = adapter;
+        this.jdbcEventLogger = logger;
 
         prepareDbEntities(excludedEntities);
         resetToDefaults();
@@ -327,7 +356,7 @@ public class DbGenerator {
                 }
             }
 
-            new DbGeneratorPostprocessor().execute(connection);
+            new DbGeneratorPostprocessor().execute(connection, getAdapter());
         }
         finally {
             connection.close();
@@ -344,7 +373,7 @@ public class DbGenerator {
         Statement statement = connection.createStatement();
 
         try {
-            QueryLogger.logQuery(sql, null);
+            jdbcEventLogger.logQuery(sql, null);
             statement.execute(sql);
             return true;
         }
@@ -354,7 +383,7 @@ public class DbGenerator {
             }
 
             failures.addFailure(new SimpleValidationFailure(sql, ex.getMessage()));
-            QueryLogger.logQueryError(ex);
+            jdbcEventLogger.logQueryError(ex);
             return false;
         }
         finally {
@@ -362,17 +391,6 @@ public class DbGenerator {
         }
     }
 
-    /**
-     * Returns an array of queries to create foreign key constraints for a particular
-     * DbEntity.
-     * 
-     * @deprecated since 3.0 as this method is used to generate both FK and UNIQUE
-     *             constraints, use 'createConstraintsQueries' instead.
-     */
-    public List<String> createFkConstraintsQueries(DbEntity table) {
-        return createConstraintsQueries(table);
-    }
-
     /**
      * Creates FK and UNIQUE constraint statements for a given table.
      * 
@@ -577,9 +595,9 @@ public class DbGenerator {
 
         // sort table list
         if (tables.size() > 1) {
-            DataNode node = new DataNode("temp");
-            node.addDataMap(map);
-            node.getEntitySorter().sortDbEntities(tables, false);
+            EntitySorter sorter = new AshwoodEntitySorter();
+            sorter.setEntityResolver(new EntityResolver(Collections.singleton(map)));
+            sorter.sortDbEntities(tables, false);
         }
 
         this.dbEntitiesInInsertOrder = tables;
