@@ -19,6 +19,7 @@
 
 package org.apache.cayenne.access;
 
+import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -55,7 +56,7 @@ import org.apache.cayenne.util.Util;
  * fetch.
  * </p>
  */
-public class IncrementalFaultList<E> implements List<E> {
+public class IncrementalFaultList<E> implements List<E>, Serializable {
 
     protected int pageSize;
     protected List elements;
@@ -76,7 +77,7 @@ public class IncrementalFaultList<E> implements List<E> {
      * Defines the upper limit on the size of fetches. This is needed to avoid where
      * clause size limitations.
      */
-    protected int maxFetchSize = 10000;
+    protected int maxFetchSize;
 
     // Don't confuse this with the JDBC ResultSet fetch size setting - this controls
     // the where clause generation that is necessary to fetch specific records a page
@@ -84,23 +85,15 @@ public class IncrementalFaultList<E> implements List<E> {
     // or complexity of the where clause - e.g., PostgreSQL having a default limit of
     // 10,000 nested expressions.
 
+    
     /**
-     * Creates a new list copying settings from another list. Elements WILL NOT be copied
-     * or fetched.
-     * 
-     * @deprecated since 3.0 - unused
+     * @deprecated since 3.1 use {@link #IncrementalFaultList(DataContext, Query, int)}.
      */
-    public IncrementalFaultList(IncrementalFaultList<E> list) {
-        this.pageSize = list.pageSize;
-        this.internalQuery = list.internalQuery;
-        this.dataContext = list.dataContext;
-        this.rootEntity = list.rootEntity;
-        this.maxFetchSize = list.maxFetchSize;
-        this.idWidth = list.idWidth;
-        this.helper = list.helper;
-        elements = Collections.synchronizedList(new ArrayList<Object>());
+    @Deprecated
+    public IncrementalFaultList(DataContext dataContext, Query query) {
+        this(dataContext, query, 10000);
     }
-
+    
     /**
      * Creates a new IncrementalFaultList using a given DataContext and query.
      * 
@@ -108,8 +101,9 @@ public class IncrementalFaultList<E> implements List<E> {
      *            objects.
      * @param query Main query used to retrieve data. Must have "pageSize" property set to
      *            a value greater than zero.
+     * @param maxFetchSize maximum number of fetches in one query
      */
-    public IncrementalFaultList(DataContext dataContext, Query query) {
+    public IncrementalFaultList(DataContext dataContext, Query query, int maxFetchSize) {
         QueryMetadata metadata = query.getMetaData(dataContext.getEntityResolver());
         if (metadata.getPageSize() <= 0) {
             throw new CayenneRuntimeException("Not a paginated query; page size: "
@@ -138,6 +132,8 @@ public class IncrementalFaultList<E> implements List<E> {
         List<Object> elementsUnsynced = new ArrayList<Object>();
         fillIn(query, elementsUnsynced);
         this.elements = Collections.synchronizedList(elementsUnsynced);
+
+        this.maxFetchSize = maxFetchSize;
     }
 
     /**
@@ -159,21 +155,6 @@ public class IncrementalFaultList<E> implements List<E> {
         return internalQuery;
     }
 
-    /**
-     * Performs initialization of the internal list of objects. Only the first page is
-     * fully resolved. For the rest of the list, only ObjectIds are read.
-     * 
-     * @deprecated since 3.0 this method is not called and is deprecated in favor of
-     *             {@link #fillIn(Query, List)}, as this method performed unneeded
-     *             synchronization.
-     * @since 1.0.6
-     */
-    protected void fillIn(Query query) {
-        synchronized (elements) {
-            fillIn(query, elements);
-        }
-    }
-
     /**
      * Performs initialization of the list of objects. Only the first page is fully
      * resolved. For the rest of the list, only ObjectIds are read.
@@ -192,10 +173,6 @@ public class IncrementalFaultList<E> implements List<E> {
                 while (it.hasNextRow()) {
                     elementsList.add(it.nextRow());
                 }
-
-                QueryLogger.logSelectCount(elementsList.size(), System
-                        .currentTimeMillis()
-                        - t1);
             }
             finally {
                 it.close();
@@ -279,8 +256,11 @@ public class IncrementalFaultList<E> implements List<E> {
 
             // fetch the range of objects in fetchSize chunks
             boolean fetchesDataRows = internalQuery.isFetchingDataRows();
-            List objects = new ArrayList(qualsSize);
-            int fetchEnd = Math.min(qualsSize, maxFetchSize);
+            List<?> objects = new ArrayList<Object>(qualsSize);
+            
+            int fetchSize = maxFetchSize > 0 ? maxFetchSize : Integer.MAX_VALUE;
+            
+            int fetchEnd = Math.min(qualsSize, fetchSize);
             int fetchBegin = 0;
             while (fetchBegin < qualsSize) {
                 SelectQuery query = new SelectQuery(rootEntity, ExpressionFactory
@@ -294,7 +274,7 @@ public class IncrementalFaultList<E> implements List<E> {
 
                 objects.addAll(dataContext.performQuery(query));
                 fetchBegin = fetchEnd;
-                fetchEnd += Math.min(maxFetchSize, qualsSize - fetchEnd);
+                fetchEnd += Math.min(fetchSize, qualsSize - fetchEnd);
             }
 
             // sanity check - database data may have changed
@@ -391,8 +371,6 @@ public class IncrementalFaultList<E> implements List<E> {
      * database. This setting governs the size/complexity of the where clause generated to
      * retrieve the next page of records. If the fetch size is less than the page size,
      * then multiple fetches will be made to resolve a page.
-     * 
-     * @return int
      */
     public int getMaxFetchSize() {
         return maxFetchSize;
@@ -650,7 +628,7 @@ public class IncrementalFaultList<E> implements List<E> {
         return unfetchedObjects;
     }
 
-    abstract class IncrementalListHelper {
+    abstract class IncrementalListHelper implements Serializable {
 
         int indexOfObject(Object object) {
             if (unresolvedSuspect(object)) {
