@@ -46,16 +46,7 @@ import org.apache.cayenne.graph.NodeCreateOperation;
 import org.apache.cayenne.graph.NodeDeleteOperation;
 import org.apache.cayenne.graph.NodeDiff;
 import org.apache.cayenne.graph.NodePropertyChangeOperation;
-import org.apache.cayenne.map.DataMap;
-import org.apache.cayenne.map.DbEntity;
-import org.apache.cayenne.map.ObjEntity;
-import org.apache.cayenne.map.Procedure;
 import org.apache.cayenne.query.ObjectIdQuery;
-import org.apache.cayenne.query.PrefetchTreeNode;
-import org.apache.cayenne.query.Query;
-import org.apache.cayenne.query.QueryCacheStrategy;
-import org.apache.cayenne.query.QueryMetadata;
-import org.apache.cayenne.query.RefreshQuery;
 import org.apache.cayenne.reflect.AttributeProperty;
 import org.apache.cayenne.reflect.ClassDescriptor;
 import org.apache.cayenne.reflect.PropertyVisitor;
@@ -72,10 +63,6 @@ import org.apache.commons.collections.map.ReferenceMap;
  * 
  * @since 1.0
  */
-// Synchronization Note: There is often a need to do double synchronize on an ObjectStore
-// and an underlying DataRowCache. To avoid deadlocks, Cayenne consistently follows the
-// policy of locking an ObjectStore first, and then locking DataRowStore. This pattern
-// must be followed in any new related developments.
 public class ObjectStore implements Serializable, SnapshotEventListener, GraphManager {
 
     /**
@@ -106,7 +93,7 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
     protected transient DataRowStore dataRowCache;
 
     // used to avoid incorrect on-demand DataRowStore initialization after deserialization
-    private boolean dataRowCacheSet;
+    protected boolean dataRowCacheSet;
 
     private Collection<GraphDiff> lifecycleEventInducedChanges;
 
@@ -115,12 +102,20 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
      */
     protected DataContext context;
 
+    /**
+     * @deprecated since 3.1
+     */
+    @Deprecated
     public ObjectStore() {
         this(null);
     }
 
+    /**
+     * @deprecated since 3.1
+     */
+    @Deprecated
     public ObjectStore(DataRowStore dataRowCache) {
-        this(dataRowCache, null);
+        this(dataRowCache, createObjectMap());
     }
 
     /**
@@ -132,7 +127,12 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
      */
     public ObjectStore(DataRowStore dataRowCache, Map<Object, Persistent> objectMap) {
         setDataRowCache(dataRowCache);
-        this.objectMap = objectMap != null ? objectMap : ObjectStore.createObjectMap();
+        if (objectMap != null) {
+            this.objectMap = objectMap;
+        }
+        else {
+            throw new CayenneRuntimeException("Object map is null.");
+        }
         this.changes = new HashMap<Object, ObjectDiff>();
     }
 
@@ -185,11 +185,12 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
         if (objectDiff == null) {
 
             Persistent object = objectMap.get(nodeId);
-            
+
             if (object == null) {
-                throw new CayenneRuntimeException("No object is registered in context with Id " + nodeId);
+                throw new CayenneRuntimeException(
+                        "No object is registered in context with Id " + nodeId);
             }
-            
+
             if (object.getPersistenceState() == PersistenceState.COMMITTED) {
                 object.setPersistenceState(PersistenceState.MODIFIED);
 
@@ -241,20 +242,6 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
         return objectMap.size();
     }
 
-    /**
-     * Returns a number of query results cached by this object store. Note that each
-     * result is a list and can possibly contain a large number of entries.
-     * 
-     * @since 1.2
-     * @deprecated since 3.0. See {@link DataContext#getQueryCache()}.
-     */
-    @Deprecated
-    public int cachedQueriesCount() {
-        return context != null && context.getQueryCache() != null ? context
-                .getQueryCache()
-                .size() : 0;
-    }
-
     /**
      * Returns a DataRowStore associated with this ObjectStore.
      */
@@ -314,25 +301,10 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
         dataRowCacheSet = dataRowCache != null;
     }
 
-    /**
-     * Invalidates a collection of DataObjects. Changes objects state to HOLLOW.
-     * 
-     * @deprecated since 3.0, use {@link DataContext#invalidateObjects(Collection)} or
-     *             {@link RefreshQuery}.
-     */
-    @Deprecated
-    public synchronized void objectsInvalidated(Collection objects) {
-        if (context != null) {
-            context.invalidateObjects(objects);
-        }
-    }
-
     /**
      * Evicts a collection of DataObjects from the ObjectStore, invalidates the underlying
      * cache snapshots. Changes objects state to TRANSIENT. This method can be used for
      * manual cleanup of Cayenne cache.
-     * 
-     * @see #objectsInvalidated(Collection)
      */
     // this method is exactly the same as "objectsInvalidated", only additionally it
     // throws out registered objects
@@ -413,31 +385,6 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
         this.changes = new HashMap<Object, ObjectDiff>();
     }
 
-    /**
-     * Updates snapshots in the underlying DataRowStore. If <code>refresh</code> is true,
-     * all snapshots in <code>snapshots</code> will be loaded into DataRowStore,
-     * regardless of the existing cache state. If <code>refresh</code> is false, only
-     * missing snapshots are loaded. This method is normally called internally by the
-     * DataContext owning the ObjectStore to update the caches after a select query.
-     * 
-     * @param objects a list of object whose snapshots need to be updated.
-     * @param snapshots a list of snapshots. Must be of the same length and use the same
-     *            order as <code>objects</code> list.
-     * @param refresh controls whether existing cached snapshots should be replaced with
-     *            the new ones.
-     * @since 1.1
-     * @deprecated since 3.0 unused
-     */
-    @Deprecated
-    public void snapshotsUpdatedForObjects(List objects, List snapshots, boolean refresh) {
-        DataRowStore cache = getDataRowCache();
-        if (cache != null) {
-            synchronized (this) {
-                cache.snapshotsUpdatedForObjects(objects, snapshots, refresh);
-            }
-        }
-    }
-
     /**
      * Builds and returns GraphDiff reflecting all uncommitted object changes.
      * 
@@ -532,32 +479,6 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
         this.changes = new HashMap<Object, ObjectDiff>();
     }
 
-    /**
-     * Starts tracking the registration of new objects from this ObjectStore. Used in
-     * conjunction with unregisterNewObjects() to control garbage collection when an
-     * instance of ObjectStore is used over a longer time for batch processing.
-     * 
-     * @deprecated since 3.0 as ObjectStore holds weak reference to unmodified objects and
-     *             this feature is useless.
-     */
-    @Deprecated
-    public synchronized void startTrackingNewObjects() {
-        // noop
-    }
-
-    /**
-     * Unregisters the newly registered DataObjects from this objectStore. Used in
-     * conjunction with startTrackingNewObjects() to control garbage collection when an
-     * instance of ObjectStore is used over a longer time for batch processing.
-     * 
-     * @deprecated since 3.0 as ObjectStore holds weak reference to unmodified objects and
-     *             this feature is useless.
-     */
-    @Deprecated
-    public synchronized void unregisterNewObjects() {
-        // noop
-    }
-
     /**
      * Returns a snapshot for ObjectId from the underlying snapshot cache. If cache
      * contains no snapshot, a null is returned.
@@ -576,34 +497,6 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
         }
     }
 
-    /**
-     * Returns cached query results for a given query, or null if no results are cached.
-     * Note that ObjectStore will only lookup results in its local cache, and not the
-     * shared cache associated with the underlying DataRowStore.
-     * 
-     * @since 1.1
-     * @deprecated since 3.0. See {@link DataContext#getQueryCache()}.
-     */
-    @Deprecated
-    public synchronized List getCachedQueryResult(String name) {
-        return context != null && context.getQueryCache() != null ? context
-                .getQueryCache()
-                .get(new CacheQueryMetadata(name)) : null;
-    }
-
-    /**
-     * Caches a list of query results.
-     * 
-     * @since 1.1
-     * @deprecated since 3.0. See {@link DataContext#getQueryCache()}.
-     */
-    @Deprecated
-    public synchronized void cacheQueryResult(String name, List results) {
-        if (context != null) {
-            context.getQueryCache().put(new CacheQueryMetadata(name), results);
-        }
-    }
-
     /**
      * Returns a snapshot for ObjectId from the underlying snapshot cache. If cache
      * contains no snapshot, it will attempt fetching it using provided QueryEngine. If
@@ -713,19 +606,6 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
         context.fireDataChannelChanged(originatingContext, diff);
     }
 
-    /**
-     * Initializes object with data from cache or from the database, if this object is not
-     * fully resolved.
-     * 
-     * @since 1.1
-     * @deprecated since 3.0 use
-     *             {@link ObjectContext#prepareForAccess(Persistent, String, boolean)}.
-     */
-    @Deprecated
-    public void resolveHollow(Persistent object) {
-        context.prepareForAccess(object, null, false);
-    }
-
     void processIdChange(Object nodeId, Object newId) {
         Persistent object = objectMap.remove(nodeId);
 
@@ -1154,106 +1034,4 @@ public class ObjectStore implements Serializable, SnapshotEventListener, GraphMa
             throw new UnsupportedOperationException();
         }
     }
-
-    /**
-     * @deprecated since 3.0 as this inner class is used to provide backwards
-     *             compatibility for some deprecated methods.
-     */
-    @Deprecated
-    final class CacheQueryMetadata implements QueryMetadata {
-
-        private String cacheKey;
-
-        CacheQueryMetadata(String cacheKey) {
-            this.cacheKey = cacheKey;
-        }
-
-        public String getCacheKey() {
-            return cacheKey;
-        }
-
-        public List<Object> getResultSetMapping() {
-            return null;
-        }
-
-        public Query getOrginatingQuery() {
-            return null;
-        }
-
-        public String[] getCacheGroups() {
-            return null;
-        }
-
-        public String getCachePolicy() {
-            return null;
-        }
-
-        public QueryCacheStrategy getCacheStrategy() {
-            return null;
-        }
-
-        public DataMap getDataMap() {
-            return null;
-        }
-
-        public DbEntity getDbEntity() {
-            return null;
-        }
-
-        public int getFetchLimit() {
-            return 0;
-        }
-
-        public int getFetchOffset() {
-            return 0;
-        }
-
-        /**
-         * @deprecated since 3.0
-         */
-        @Deprecated
-        public int getFetchStartIndex() {
-            return getFetchOffset();
-        }
-
-        public ObjEntity getObjEntity() {
-            return null;
-        }
-
-        public ClassDescriptor getClassDescriptor() {
-            return null;
-        }
-
-        public int getPageSize() {
-            return 0;
-        }
-
-        public PrefetchTreeNode getPrefetchTree() {
-            return null;
-        }
-
-        public Map<String, String> getPathSplitAliases() {
-            return null;
-        }
-
-        public Procedure getProcedure() {
-            return null;
-        }
-
-        public boolean isFetchingDataRows() {
-            return false;
-        }
-
-        public boolean isRefreshingObjects() {
-            return false;
-        }
-
-        public boolean isResolvingInherited() {
-            return false;
-        }
-
-        public int getStatementFetchSize() {
-            return 0;
-        }
-    }
 }
