@@ -43,6 +43,7 @@ import org.apache.cayenne.ejbql.parser.EJBQLSubselect;
 import org.apache.cayenne.ejbql.parser.EJBQLTrimBoth;
 import org.apache.cayenne.ejbql.parser.EJBQLTrimSpecification;
 import org.apache.cayenne.ejbql.parser.Node;
+import org.apache.cayenne.ejbql.parser.SimpleNode;
 import org.apache.cayenne.map.DbAttribute;
 import org.apache.cayenne.map.DbEntity;
 import org.apache.cayenne.map.DbJoin;
@@ -385,11 +386,11 @@ public class EJBQLConditionTranslator extends EJBQLBaseVisitor {
     }
     
     /**
-     * Checks expression for containing null imput parameter.
+     * Checks expression for containing null input parameter.
      * For that, we'll append IS NULL or IS NOT NULL instead of =null or <>null
      * @return whether replacement was done and there's no need for normal expression processing
      */
-    boolean checkNullParameter(EJBQLExpression expression, String toAppend) {
+    protected boolean checkNullParameter(EJBQLExpression expression, String toAppend) {
         if (expression.getChildrenCount() == 2) {
             // We rewrite expression "parameter = :x" where x=null
             // as "parameter IS NULL"
@@ -421,7 +422,7 @@ public class EJBQLConditionTranslator extends EJBQLBaseVisitor {
     public boolean visitEquals(EJBQLExpression expression, int finishedChildIndex) {
         switch (finishedChildIndex) {
             case 0:
-            	if (checkNullParameter(expression, " IS NULL")) {
+                if (checkNullParameter(expression, " IS NULL")) {
                     return false;
                 }
                 context.append(" =");
@@ -481,7 +482,7 @@ public class EJBQLConditionTranslator extends EJBQLBaseVisitor {
     public boolean visitNotEquals(EJBQLExpression expression, int finishedChildIndex) {
         switch (finishedChildIndex) {
             case 0:
-            	if (checkNullParameter(expression, " IS NOT NULL")) {
+                if (checkNullParameter(expression, " IS NOT NULL")) {
                     return false;
                 }
                 context.append(" <>");
@@ -562,6 +563,10 @@ public class EJBQLConditionTranslator extends EJBQLBaseVisitor {
     @Override
     public boolean visitLike(EJBQLExpression expression, int finishedChildIndex) {
         if (finishedChildIndex == 0) {
+            if (checkNullParameter(expression, " IS NULL")) {
+                return false;
+            }
+            
             if (expression.isNegated()) {
                 context.append(" NOT");
             }
@@ -841,7 +846,7 @@ public class EJBQLConditionTranslator extends EJBQLBaseVisitor {
         else {
 
             String type = null;
-            EJBQLEquals parent = ((EJBQLNamedInputParameter) expression).getParent();
+            Node parent = ((SimpleNode) expression).jjtGetParent();
 
             context.pushMarker("@processParameter", true);
 
