@@ -30,17 +30,17 @@ import org.apache.cayenne.exp.Expression;
 import org.apache.cayenne.exp.TraversalHandler;
 import org.apache.cayenne.exp.parser.ASTDbPath;
 import org.apache.cayenne.exp.parser.ASTObjPath;
+import org.apache.cayenne.exp.parser.PatternMatchNode;
 import org.apache.cayenne.exp.parser.SimpleNode;
 import org.apache.cayenne.map.DbAttribute;
 import org.apache.cayenne.map.DbRelationship;
-import org.apache.cayenne.map.EntityInheritanceTree;
 import org.apache.cayenne.map.JoinType;
 import org.apache.cayenne.map.ObjEntity;
 import org.apache.cayenne.query.QualifiedQuery;
 import org.apache.cayenne.query.Query;
+import org.apache.cayenne.reflect.ClassDescriptor;
 import org.apache.commons.collections.IteratorUtils;
 import org.apache.commons.collections.Transformer;
-import org.apache.cayenne.exp.parser.PatternMatchNode;
 
 /**
  * Translates query qualifier to SQL. Used as a helper class by query translators.
@@ -49,9 +49,12 @@ public class QualifierTranslator extends QueryAssemblerHelper implements Travers
 
     protected DataObjectMatchTranslator objectMatchTranslator;
     protected boolean matchingObject;
+    protected boolean caseInsensitive;
 
     public QualifierTranslator(QueryAssembler queryAssembler) {
         super(queryAssembler);
+
+        caseInsensitive = false;
     }
 
     /**
@@ -64,9 +67,14 @@ public class QualifierTranslator extends QueryAssemblerHelper implements Travers
     protected void doAppendPart() throws IOException {
         doAppendPart(extractQualifier());
     }
-    
+
+    public void setCaseInsensitive(boolean caseInsensitive) {
+        this.caseInsensitive = caseInsensitive;
+    }
+
     /**
-     * Translates query qualifier to SQL WHERE clause. Qualifier is a method parameter     * 
+     * Translates query qualifier to SQL WHERE clause. Qualifier is a method parameter.
+     * 
      * @since 3.0
      */
     protected void doAppendPart(Expression rootNode) throws IOException {
@@ -85,18 +93,20 @@ public class QualifierTranslator extends QueryAssemblerHelper implements Travers
         ObjEntity entity = getObjEntity();
 
         if (entity != null) {
-            EntityInheritanceTree tree = queryAssembler
+
+            ClassDescriptor descriptor = queryAssembler
                     .getEntityResolver()
-                    .lookupInheritanceTree(entity);
-            Expression entityQualifier = (tree != null) ? tree
-                    .qualifierForEntityAndSubclasses() : entity.getDeclaredQualifier();
+                    .getClassDescriptor(entity.getName());
+            Expression entityQualifier = descriptor
+                    .getEntityInheritanceTree()
+                    .qualifierForEntityAndSubclasses();
             if (entityQualifier != null) {
                 qualifier = (qualifier != null)
                         ? qualifier.andExp(entityQualifier)
                         : entityQualifier;
             }
         }
-        
+
         /**
          * Attaching root Db entity's qualifier
          */
@@ -104,9 +114,9 @@ public class QualifierTranslator extends QueryAssemblerHelper implements Travers
             Expression dbQualifier = getDbEntity().getQualifier();
             if (dbQualifier != null) {
                 dbQualifier = dbQualifier.transform(new DbEntityQualifierTransformer());
-                
-                qualifier = qualifier == null ? dbQualifier :
-                    qualifier.andExp(dbQualifier);
+
+                qualifier = qualifier == null ? dbQualifier : qualifier
+                        .andExp(dbQualifier);
             }
         }
 
@@ -250,10 +260,20 @@ public class QualifierTranslator extends QueryAssemblerHelper implements Travers
                     out.append(" NOT LIKE ");
                     break;
                 case Expression.LIKE_IGNORE_CASE:
-                    out.append(") LIKE UPPER(");
+                    if (caseInsensitive) {
+                        out.append(" LIKE ");
+                    }
+                    else {
+                        out.append(") LIKE UPPER(");
+                    }
                     break;
                 case Expression.NOT_LIKE_IGNORE_CASE:
-                    out.append(") NOT LIKE UPPER(");
+                    if (caseInsensitive) {
+                        out.append(" NOT LIKE ");
+                    }
+                    else {
+                        out.append(") NOT LIKE UPPER(");
+                    }
                     break;
                 case Expression.ADD:
                     out.append(" + ");
@@ -279,6 +299,15 @@ public class QualifierTranslator extends QueryAssemblerHelper implements Travers
                     else if (childIndex == 1)
                         out.append(" AND ");
                     break;
+                case Expression.BITWISE_OR:
+                    out.append(" ").append(operandForBitwiseOr()).append(" ");
+                    break;
+                case Expression.BITWISE_AND:
+                    out.append(" ").append(operandForBitwiseAnd()).append(" ");
+                    break;
+                case Expression.BITWISE_XOR:
+                    out.append(" ").append(operandForBitwiseXor()).append(" ");
+                    break;
             }
         }
         catch (IOException ioex) {
@@ -291,6 +320,34 @@ public class QualifierTranslator extends QueryAssemblerHelper implements Travers
         }
     }
 
+    /**
+     * @since 3.1
+     */
+    protected String operandForBitwiseNot() {
+        return "~";
+    }
+
+    /**
+     * @since 3.1
+     */
+    protected String operandForBitwiseOr() {
+        return "|";
+    }
+
+    /**
+     * @since 3.1
+     */
+    protected String operandForBitwiseAnd() {
+        return "&";
+    }
+    
+    /**
+     * @since 3.1
+     */
+    protected String operandForBitwiseXor() {
+        return "^";
+    }
+
     public void startNode(Expression node, Expression parentNode) {
         int count = node.getOperandCount();
 
@@ -323,9 +380,12 @@ public class QualifierTranslator extends QueryAssemblerHelper implements Travers
                 // qualBuf.append('+');
                 else if (node.getType() == Expression.NOT)
                     out.append("NOT ");
+                else if (node.getType() == Expression.BITWISE_NOT) {
+                    out.append(operandForBitwiseNot());
+                }
             }
-            else if (node.getType() == Expression.LIKE_IGNORE_CASE
-                    || node.getType() == Expression.NOT_LIKE_IGNORE_CASE) {
+            else if ((node.getType() == Expression.LIKE_IGNORE_CASE || node.getType() == Expression.NOT_LIKE_IGNORE_CASE)
+                    && !caseInsensitive) {
                 out.append("UPPER(");
             }
         }
@@ -352,17 +412,19 @@ public class QualifierTranslator extends QueryAssemblerHelper implements Travers
             boolean isPatternMatchNode = PatternMatchNode.class.isAssignableFrom(node
                     .getClass());
 
-            if (isPatternMatchNode && !likeIgnoreCase)
-                appendLikeEscapeCharacter((PatternMatchNode) node);
-
-            if (parenthesisNeeded)
+            // closing UPPER parenthesis
+            if (likeIgnoreCase && !caseInsensitive) {
                 out.append(')');
+            }
 
-            if (isPatternMatchNode && likeIgnoreCase)
+            if (isPatternMatchNode) {
                 appendLikeEscapeCharacter((PatternMatchNode) node);
+            }
 
-            if (likeIgnoreCase)
+            // closing LIKE parenthesis
+            if (parenthesisNeeded) {
                 out.append(')');
+            }
         }
         catch (IOException ioex) {
             throw new CayenneRuntimeException("Error appending content", ioex);
@@ -472,12 +534,14 @@ public class QualifierTranslator extends QueryAssemblerHelper implements Travers
             objectMatchTranslator.setRelationship(rel, joinSplitAlias);
         }
     }
-    
+
     /**
-     * Class to translate DB Entity qualifiers annotation to Obj-entity qualifiers annotation
-     * This is done by changing all Obj-paths to Db-paths and rejecting all original Db-paths
+     * Class to translate DB Entity qualifiers annotation to Obj-entity qualifiers
+     * annotation This is done by changing all Obj-paths to Db-paths and rejecting all
+     * original Db-paths
      */
     class DbEntityQualifierTransformer implements Transformer {
+
         public Object transform(Object input) {
             if (input instanceof ASTObjPath) {
                 return new ASTDbPath(((SimpleNode) input).getOperand(0));
