@@ -19,16 +19,6 @@
 
 package org.apache.cayenne.access.trans;
 
-import java.sql.Types;
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-
 import org.apache.cayenne.CayenneRuntimeException;
 import org.apache.cayenne.access.jdbc.ColumnDescriptor;
 import org.apache.cayenne.dba.QuotingStrategy;
@@ -55,6 +45,16 @@ import org.apache.cayenne.util.CayenneMapEntry;
 import org.apache.cayenne.util.EqualsBuilder;
 import org.apache.cayenne.util.HashCodeBuilder;
 
+import java.sql.Types;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+
 /**
  * A builder of JDBC PreparedStatements based on Cayenne SelectQueries. Translates
  * SelectQuery to parameterized SQL string and wraps it in a PreparedStatement.
@@ -174,15 +174,7 @@ public class SelectTranslator extends QueryAssembler {
             }
         }
 
-        // append columns (unroll the loop's first element)
-        int columnCount = selectColumnExpList.size();
-        queryBuf.append(selectColumnExpList.get(0));
-
-        // assume there is at least 1 element
-        for (int i = 1; i < columnCount; i++) {
-            queryBuf.append(", ");
-            queryBuf.append(selectColumnExpList.get(i));
-        }
+        appendSelectColumns(queryBuf, selectColumnExpList);
 
         // append from clause
         queryBuf.append(" FROM ");
@@ -213,6 +205,22 @@ public class SelectTranslator extends QueryAssembler {
 
         return queryBuf.toString();
     }
+    
+    /**
+     * @since 3.1
+     */
+    protected void appendSelectColumns(StringBuilder buffer, List<String> selectColumnExpList) {
+        
+        // append columns (unroll the loop's first element)
+        int columnCount = selectColumnExpList.size();
+        buffer.append(selectColumnExpList.get(0));
+
+        // assume there is at least 1 element
+        for (int i = 1; i < columnCount; i++) {
+            buffer.append(", ");
+            buffer.append(selectColumnExpList.get(i));
+        }
+    }
 
     /**
      * Handles appending optional limit and offset clauses. This implementation does
@@ -342,7 +350,7 @@ public class SelectTranslator extends QueryAssembler {
                     }
                     else if (pathPart instanceof DbRelationship) {
                         DbRelationship rel = (DbRelationship) pathPart;
-                        dbRelationshipAdded(rel, JoinType.INNER, null);
+                        dbRelationshipAdded(rel, JoinType.LEFT_OUTER, null);
                     }
                     else if (pathPart instanceof DbAttribute) {
                         DbAttribute dbAttr = (DbAttribute) pathPart;
@@ -408,8 +416,14 @@ public class SelectTranslator extends QueryAssembler {
                         .resolvePath(pathExp, getPathAliases())) {
 
                     if (component.getRelationship() != null) {
-                        dbRelationshipAdded(component.getRelationship(), component
-                                .getJoinType(), null);
+                        // do not invoke dbRelationshipAdded(), invoke
+                        // pushJoin() instead. This is to prevent
+                        // 'forcingDistinct' flipping to true, that will result
+                        // in unneeded extra processing and sometimes in invalid
+                        // results (see CAY-1979). Distinctness of each row is
+                        // guaranteed by the prefetch query semantics - we
+                        // include target ID in the result columns
+                        getJoinStack().pushJoin(component.getRelationship(), component.getJoinType(), null);
                     }
 
                     lastComponent = component;
@@ -566,19 +580,21 @@ public class SelectTranslator extends QueryAssembler {
             // record ObjAttribute override
             for (ColumnDescriptor column : columns) {
                 if (attribute.getName().equals(column.getName())) {
+                    
+                    if (attributeOverrides == null) {
+                        attributeOverrides = new HashMap<ObjAttribute, ColumnDescriptor>();
+                    }
 
                     // kick out the original attribute
                     ObjAttribute original = defaultAttributesByColumn.remove(column);
 
                     if (original != null) {
-                        if (attributeOverrides == null) {
-                            attributeOverrides = new HashMap<ObjAttribute, ColumnDescriptor>();
-                        }
-
                         attributeOverrides.put(original, column);
-                        column.setJavaClass(Void.TYPE.getName());
                     }
 
+                    attributeOverrides.put(objAttribute, column);
+                    column.setJavaClass(Void.TYPE.getName());
+                    
                     break;
                 }
             }
