@@ -20,11 +20,16 @@ package org.apache.cayenne.cache;
 
 import java.util.Map;
 
+import org.apache.cayenne.di.Provider;
+
 /**
  * Creates QueryCache.
  * 
  * @since 3.0
+ * @deprecated since 3.1 QueryCacheFactory is unused and replaced by corresponding
+ *             {@link Provider} implementation for various types of QueryCache.
  */
+@Deprecated
 public interface QueryCacheFactory {
 
     /**
