@@ -19,25 +19,29 @@
 
 package org.apache.cayenne.conn;
 
-import java.io.PrintWriter;
-import java.sql.Connection;
-import java.sql.SQLException;
-import java.util.LinkedList;
-import java.util.List;
-import java.util.ListIterator;
+import org.apache.cayenne.CayenneRuntimeException;
+import org.apache.cayenne.di.ScopeEventListener;
+import org.apache.cayenne.log.JdbcEventLogger;
 
 import javax.sql.ConnectionEvent;
 import javax.sql.ConnectionEventListener;
 import javax.sql.ConnectionPoolDataSource;
 import javax.sql.DataSource;
 import javax.sql.PooledConnection;
+import java.io.PrintWriter;
+import java.sql.Connection;
+import java.sql.SQLException;
+import java.sql.SQLFeatureNotSupportedException;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.ListIterator;
+import java.util.logging.Logger;
 
 /**
- * PoolManager is a pooling DataSource impementation. Internally to obtain connections
- * PoolManager uses either a JDBC driver or another pooling datasource.
- * 
+ * PoolManager is a Cayenne implementation of a pooling DataSource.
  */
-public class PoolManager implements DataSource, ConnectionEventListener {
+public class PoolManager implements ScopeEventListener, DataSource,
+        ConnectionEventListener {
 
     /**
      * Defines a maximum time in milliseconds that a connection request could wait in the
@@ -59,6 +63,8 @@ public class PoolManager implements DataSource, ConnectionEventListener {
 
     private PoolMaintenanceThread poolMaintenanceThread;
 
+    private boolean shuttingDown;
+
     /**
      * Creates new PoolManager using org.apache.cayenne.conn.PoolDataSource for an
      * underlying ConnectionPoolDataSource.
@@ -70,8 +76,7 @@ public class PoolManager implements DataSource, ConnectionEventListener {
     }
 
     public PoolManager(String jdbcDriver, String dataSourceUrl, int minCons, int maxCons,
-            String userName, String password, ConnectionEventLoggingDelegate logger)
-            throws SQLException {
+            String userName, String password, JdbcEventLogger logger) throws SQLException {
 
         if (logger != null) {
             DataSourceInfo info = new DataSourceInfo();
@@ -168,38 +173,74 @@ public class PoolManager implements DataSource, ConnectionEventListener {
         return connection;
     }
 
-    /** Closes all existing connections, removes them from the pool. */
+    /**
+     * Closes all existing connections, removes them from the pool.
+     * 
+     * @deprecated since 3.1 replaced with {@link #shutdown()} method for naming
+     *             consistency.
+     */
+    @Deprecated
     public void dispose() throws SQLException {
-        synchronized (this) {
-            // clean connections from the pool
-            ListIterator<PooledConnection> unusedIterator = unusedPool.listIterator();
-            while (unusedIterator.hasNext()) {
-                PooledConnection con = unusedIterator.next();
-                // close connection
-                con.close();
-                // remove connection from the list
-                unusedIterator.remove();
-            }
+        shutdown();
+    }
 
-            // clean used connections
-            ListIterator<PooledConnection> usedIterator = usedPool.listIterator();
-            while (usedIterator.hasNext()) {
-                PooledConnection con = usedIterator.next();
-                // stop listening for connection events
-                con.removeConnectionEventListener(this);
-                // close connection
-                con.close();
-                // remove connection from the list
-                usedIterator.remove();
-            }
+    /**
+     * Closes all existing connections, drains the pool and stops the maintenance thread.
+     * 
+     * @since 3.1
+     */
+    public synchronized void shutdown() throws SQLException {
+
+        // disposing maintenance thread first to avoid any changes to pools
+        // during shutdown
+        disposeOfMaintenanceThread();
+
+        // using boolean variable instead of locking PoolManager instance due to
+        // possible deadlock during shutdown when one of connections locks its
+        // event listeners list trying to invoke locked PoolManager's listener methods
+        shuttingDown = true;
+
+        ListIterator<PooledConnection> unusedIterator = unusedPool.listIterator();
+        while (unusedIterator.hasNext()) {
+            PooledConnection con = unusedIterator.next();
+            // close connection
+            con.close();
+            // remove connection from the list
+            unusedIterator.remove();
         }
 
-        disposeOfMaintenanceThread();
+        // clean used connections
+        ListIterator<PooledConnection> usedIterator = usedPool.listIterator();
+        while (usedIterator.hasNext()) {
+            PooledConnection con = usedIterator.next();
+            // stop listening for connection events
+            con.removeConnectionEventListener(this);
+            // close connection
+            con.close();
+            // remove connection from the list
+            usedIterator.remove();
+        }
+    }
+
+    /**
+     * An implementation of {@link ScopeEventListener} that simply calls
+     * {@link #shutdown()}.
+     * 
+     * @since 3.1
+     */
+    public void beforeScopeEnd() {
+        try {
+            shutdown();
+        }
+        catch (SQLException e) {
+            throw new CayenneRuntimeException("Error while shutting down");
+        }
     }
 
     protected void disposeOfMaintenanceThread() {
         if (poolMaintenanceThread != null) {
-            this.poolMaintenanceThread.dispose();
+            poolMaintenanceThread.shutdown();
+            poolMaintenanceThread = null;
         }
     }
 
@@ -321,7 +362,7 @@ public class PoolManager implements DataSource, ConnectionEventListener {
 
     /**
      * Returns connection from the pool using internal values of user name and password.
-     * Eqivalent to calling:
+     * Equivalent to calling:
      * <p>
      * <code>ds.getConnection(ds.getUserName(), ds.getPassword())</code>
      * </p>
@@ -334,6 +375,10 @@ public class PoolManager implements DataSource, ConnectionEventListener {
     public synchronized Connection getConnection(String userName, String password)
             throws SQLException {
 
+        if (shuttingDown) {
+            throw new SQLException("Pool manager is shutting down.");
+        }
+
         PooledConnection pooledConnection = uncheckPooledConnection(userName, password);
 
         try {
@@ -434,6 +479,11 @@ public class PoolManager implements DataSource, ConnectionEventListener {
      * Returns closed connection to the pool.
      */
     public synchronized void connectionClosed(ConnectionEvent event) {
+
+        if (shuttingDown) {
+            return;
+        }
+
         // return connection to the pool
         PooledConnection closedConn = (PooledConnection) event.getSource();
 
@@ -459,6 +509,11 @@ public class PoolManager implements DataSource, ConnectionEventListener {
      * is in invalid state.
      */
     public synchronized void connectionErrorOccurred(ConnectionEvent event) {
+
+        if (shuttingDown) {
+            return;
+        }
+
         // later on we should analyze the error to see if this
         // is fatal... right now just kill this PooledConnection
 
@@ -484,8 +539,8 @@ public class PoolManager implements DataSource, ConnectionEventListener {
 
     static class PoolMaintenanceThread extends Thread {
 
-        protected boolean shouldDie;
-        protected PoolManager pool;
+        private boolean shouldDie;
+        private PoolManager pool;
 
         PoolMaintenanceThread(PoolManager pool) {
             super.setName("PoolManagerCleanup-" + pool.hashCode());
@@ -506,15 +561,15 @@ public class PoolManager implements DataSource, ConnectionEventListener {
                     // ignore...
                 }
 
-                if (shouldDie) {
-                    break;
-                }
-
                 synchronized (pool) {
                     // TODO: implement a smarter algorithm for pool management...
                     // right now it will simply close one connection if the count is
                     // above median and there are any idle connections.
 
+                    if (shouldDie) {
+                        break;
+                    }
+
                     int unused = pool.getCurrentlyUnused();
                     int used = pool.getCurrentlyInUse();
                     int total = unused + used;
@@ -531,10 +586,11 @@ public class PoolManager implements DataSource, ConnectionEventListener {
         }
 
         /**
-         * Stops the thread.
+         * Stops the maintenance thread.
          */
-        public void dispose() {
+        void shutdown() {
             shouldDie = true;
+            interrupt();
         }
     }
 
@@ -553,4 +609,11 @@ public class PoolManager implements DataSource, ConnectionEventListener {
     public <T> T unwrap(Class<T> iface) throws SQLException {
         throw new UnsupportedOperationException();
     }
+
+    /**
+     * @since 3.1 JDBC 4.1 compatibility under Java 1.5
+     */
+    public Logger getParentLogger() throws SQLFeatureNotSupportedException {
+        throw new UnsupportedOperationException();
+    }
 }
