@@ -19,6 +19,8 @@
 
 package org.apache.cayenne.dba;
 
+import org.apache.cayenne.util.Util;
+
 import java.io.Serializable;
 import java.math.BigInteger;
 import java.sql.DatabaseMetaData;
@@ -32,12 +34,9 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
-import org.apache.cayenne.util.Util;
-
 /**
  * A utility class that handles mappings of JDBC data types to the database types and Java
  * types. Also contains methods that provide information about JDBC types.
- * 
  */
 public class TypesMapping {
 
@@ -184,7 +183,7 @@ public class TypesMapping {
         sqlEnumJava.put(Integer.valueOf(Types.NUMERIC), JAVA_BIGDECIMAL);
         sqlEnumJava.put(Integer.valueOf(Types.REAL), JAVA_FLOAT);
         sqlEnumJava.put(Integer.valueOf(Types.SMALLINT), JAVA_SHORT);
-        sqlEnumJava.put(Integer.valueOf(Types.TINYINT), JAVA_BYTE);
+        sqlEnumJava.put(Integer.valueOf(Types.TINYINT), JAVA_SHORT);
         sqlEnumJava.put(Integer.valueOf(Types.TIME), JAVA_UTILDATE);
         sqlEnumJava.put(Integer.valueOf(Types.TIMESTAMP), JAVA_UTILDATE);
         sqlEnumJava.put(Integer.valueOf(Types.VARBINARY), JAVA_BYTES);
@@ -202,7 +201,7 @@ public class TypesMapping {
         javaSqlEnum.put(JAVA_FLOAT, Integer.valueOf(Types.FLOAT));
         javaSqlEnum.put(JAVA_INTEGER, Integer.valueOf(Types.INTEGER));
         javaSqlEnum.put(JAVA_SHORT, Integer.valueOf(Types.SMALLINT));
-        javaSqlEnum.put(JAVA_BYTE, Integer.valueOf(Types.TINYINT));
+        javaSqlEnum.put(JAVA_BYTE, Integer.valueOf(Types.SMALLINT));
         javaSqlEnum.put(JAVA_TIME, Integer.valueOf(Types.TIME));
         javaSqlEnum.put(JAVA_TIMESTAMP, Integer.valueOf(Types.TIMESTAMP));
 
@@ -220,17 +219,11 @@ public class TypesMapping {
     /**
      * Returns true if supplied type can have a length attribute as a part of column
      * definition.
+     *
+     * @deprecated
      */
     public static boolean supportsLength(int type) {
-        return type == Types.BINARY
-                || type == Types.CHAR
-                || type == Types.DECIMAL
-                || type == Types.DOUBLE
-                || type == Types.FLOAT
-                || type == Types.NUMERIC
-                || type == Types.REAL
-                || type == Types.VARBINARY
-                || type == Types.VARCHAR;
+        return JdbcAdapter.supportsLength(type);
     }
 
     /**
@@ -260,8 +253,8 @@ public class TypesMapping {
                 || type == Types.NUMERIC;
     }
 
-    /** 
-     * Returns an array of string names of the default JDBC data types. 
+    /**
+     * Returns an array of string names of the default JDBC data types.
      */
     public static String[] getDatabaseTypes() {
         Collection<String> types = sqlStringType.keySet();
@@ -453,20 +446,25 @@ public class TypesMapping {
      * etc.
      * 
      * @return Fully qualified Java type name or null if not found.
+     * @deprecated since 3.0. use getJavaBySqlType(int) instead. Not usable since "0" can
+     *             mean "undefined", not really zero.
      */
+    @Deprecated
     public static String getJavaBySqlType(int type, int length, int scale) {
 
         // this does not always produce the correct result. See for instance CAY-1052 - PG
         // drivers from 8.2 and newer decided that the scale of "0" means "undefined", not
         // really zero.
-        if (type == Types.NUMERIC && scale == 0) { //SQL integer's max value is 2^31-1, which has length of 10 
+        if (type == Types.NUMERIC && scale == 0) { // SQL integer's max value is 2^31-1,
+                                                   // which has length of 10
             if (length < 10) {
                 type = Types.INTEGER;
             }
-            else if (length < 19) { //SQL's BIGINT max value is 2^63-1, which has length of 19
+            else if (length < 19) { // SQL's BIGINT max value is 2^63-1, which has length
+                                    // of 19
                 type = Types.BIGINT;
             }
-            
+
         }
 
         return sqlEnumJava.get(Integer.valueOf(type));
