@@ -22,18 +22,27 @@ package org.apache.cayenne.dba.db2;
 import java.sql.DatabaseMetaData;
 import java.sql.SQLException;
 
+import org.apache.cayenne.configuration.server.DbAdapterDetector;
 import org.apache.cayenne.dba.DbAdapter;
 import org.apache.cayenne.dba.DbAdapterFactory;
+import org.apache.cayenne.di.AdhocObjectFactory;
+import org.apache.cayenne.di.Inject;
 
 /**
  * @since 1.2
  */
-public class DB2Sniffer implements DbAdapterFactory {
+public class DB2Sniffer implements DbAdapterFactory, DbAdapterDetector {
+    
+    protected AdhocObjectFactory objectFactory;
+    
+    public DB2Sniffer(@Inject AdhocObjectFactory objectFactory) {
+        this.objectFactory = objectFactory;
+    }
 
     public DbAdapter createAdapter(DatabaseMetaData md) throws SQLException {
         String dbName = md.getDatabaseProductName();
         return dbName != null && dbName.toUpperCase().contains("DB2")
-                ? new DB2Adapter()
+                ? (DbAdapter)objectFactory.newInstance(DbAdapter.class, DB2Adapter.class.getName())
                 : null;
     }
 }
