@@ -21,18 +21,28 @@ package org.apache.cayenne.dba.h2;
 
 import java.sql.DatabaseMetaData;
 import java.sql.SQLException;
+
+import org.apache.cayenne.configuration.server.DbAdapterDetector;
 import org.apache.cayenne.dba.DbAdapter;
 import org.apache.cayenne.dba.DbAdapterFactory;
+import org.apache.cayenne.di.AdhocObjectFactory;
+import org.apache.cayenne.di.Inject;
 
 /**
  * @since 3.0
  */
-public class H2Sniffer implements DbAdapterFactory {
+public class H2Sniffer implements DbAdapterFactory, DbAdapterDetector {
+    
+    protected AdhocObjectFactory objectFactory;
+    
+    public H2Sniffer(@Inject AdhocObjectFactory objectFactory) {
+        this.objectFactory = objectFactory;
+    }
 
     public DbAdapter createAdapter(DatabaseMetaData md) throws SQLException {
         String dbName = md.getDatabaseProductName();
         return dbName != null && dbName.toUpperCase().contains("H2")
-                ? new H2Adapter()
+                ? (DbAdapter)objectFactory.newInstance(DbAdapter.class, H2Adapter.class.getName())
                 : null;
     }
 
