@@ -46,7 +46,7 @@ import org.apache.cayenne.access.OperationObserver;
 import org.apache.cayenne.access.jdbc.RowDescriptorBuilder;
 import org.apache.cayenne.access.jdbc.SQLStatement;
 import org.apache.cayenne.access.jdbc.SQLTemplateAction;
-import org.apache.cayenne.dba.DbAdapter;
+import org.apache.cayenne.dba.JdbcAdapter;
 import org.apache.cayenne.map.DbEntity;
 import org.apache.cayenne.map.EntityResolver;
 import org.apache.cayenne.query.SQLTemplate;
@@ -58,7 +58,7 @@ class OracleSQLTemplateAction extends SQLTemplateAction {
 
     protected DbEntity dbEntity;
 
-    OracleSQLTemplateAction(SQLTemplate query, DbAdapter adapter,
+    OracleSQLTemplateAction(SQLTemplate query, JdbcAdapter adapter,
             EntityResolver entityResolver) {
         super(query, adapter, entityResolver);
         this.dbEntity = query.getMetaData(entityResolver).getDbEntity();
@@ -171,6 +171,7 @@ class OracleSQLTemplateAction extends SQLTemplateAction {
         /**
          * @deprecated
          */
+        @Deprecated
         public BigDecimal getBigDecimal(int columnIndex, int scale) throws SQLException {
             return delegate.getBigDecimal(columnIndex, scale);
         }
@@ -182,6 +183,7 @@ class OracleSQLTemplateAction extends SQLTemplateAction {
         /**
          * @deprecated
          */
+        @Deprecated
         public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException {
             return delegate.getBigDecimal(columnName, scale);
         }
@@ -400,6 +402,7 @@ class OracleSQLTemplateAction extends SQLTemplateAction {
         /**
          * @deprecated
          */
+        @Deprecated
         public InputStream getUnicodeStream(int columnIndex) throws SQLException {
             return delegate.getUnicodeStream(columnIndex);
         }
@@ -407,6 +410,7 @@ class OracleSQLTemplateAction extends SQLTemplateAction {
         /**
          * @deprecated
          */
+        @Deprecated
         public InputStream getUnicodeStream(String columnName) throws SQLException {
             return delegate.getUnicodeStream(columnName);
         }
@@ -1100,6 +1104,24 @@ class OracleSQLTemplateAction extends SQLTemplateAction {
                 throws SQLException {
             throw new UnsupportedOperationException();
         }
+
+        /**
+         * @since 3.1
+         *
+         * JDBC 4.1 compatibility under Java 1.5
+         */
+        public <T> T getObject(int columnIndex, Class<T> type) throws SQLException {
+            throw new UnsupportedOperationException();
+        }
+
+        /**
+         * @since 3.1
+         *
+         * JDBC 4.1 compatibility under Java 1.5
+         */
+        public <T> T getObject(String columnLabel, Class<T> type) throws SQLException {
+            throw new UnsupportedOperationException();
+        }
     }
 
     final class OracleResultSetMetadata implements ResultSetMetaData {
