@@ -22,7 +22,6 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -35,6 +34,7 @@ import org.apache.cayenne.ejbql.EJBQLExpressionVisitor;
 import org.apache.cayenne.map.DbAttribute;
 import org.apache.cayenne.map.DbJoin;
 import org.apache.cayenne.map.DbRelationship;
+import org.apache.cayenne.map.Entity;
 import org.apache.cayenne.map.EntityResolver;
 import org.apache.cayenne.map.EntityResult;
 import org.apache.cayenne.map.ObjAttribute;
@@ -71,7 +71,7 @@ class Compiler {
     private EJBQLExpressionVisitor rootDescriptorVisitor;
     private List<Object> resultComponents;
     private PrefetchTreeNode prefetchTree = null;
-    
+
     Compiler(EntityResolver resolver) {
         this.resolver = resolver;
         this.descriptorsById = new HashMap<String, ClassDescriptor>();
@@ -87,50 +87,49 @@ class Compiler {
         parsed.visit(new CompilationVisitor());
 
         Map<EJBQLPath, Integer> pathsInSelect = new HashMap<EJBQLPath, Integer>();
-       
-        if (parsed != null) {
-            for (int i = 0; i < parsed.getChildrenCount(); i++) {
-                if (parsed.getChild(i) instanceof EJBQLSelectClause) {
 
-                    EJBQLExpression parsedTemp = parsed.getChild(i);
-                    boolean stop = false;
+        for (int i = 0; i < parsed.getChildrenCount(); i++) {
+            if (parsed.getChild(i) instanceof EJBQLSelectClause) {
 
-                    while (parsedTemp.getChildrenCount() > 0 && !stop) {
-                        EJBQLExpression newParsedTemp = null;
-                        for (int j = 0; j < parsedTemp.getChildrenCount(); j++) {
-                            if (parsedTemp.getChild(j) instanceof EJBQLSelectExpression) {
-                                for (int k = 0; k < parsedTemp
-                                        .getChild(j)
-                                        .getChildrenCount(); k++) {
+                EJBQLExpression parsedTemp = parsed.getChild(i);
+                boolean stop = false;
 
-                                    if (parsedTemp.getChild(j).getChild(k) instanceof EJBQLPath) {
-                                        pathsInSelect.put((EJBQLPath) parsedTemp
-                                                .getChild(j)
-                                                .getChild(k), j);
+                while (parsedTemp.getChildrenCount() > 0 && !stop) {
+                    EJBQLExpression newParsedTemp = null;
+                    for (int j = 0; j < parsedTemp.getChildrenCount(); j++) {
+                        if (parsedTemp.getChild(j) instanceof EJBQLSelectExpression) {
+                            for (int k = 0; k < parsedTemp
+                                    .getChild(j)
+                                    .getChildrenCount(); k++) {
+
+                                if (parsedTemp.getChild(j).getChild(k) instanceof EJBQLPath) {
+                                    pathsInSelect.put((EJBQLPath) parsedTemp
+                                            .getChild(j)
+                                            .getChild(k), j);
 
-                                    }
                                 }
                             }
-                            else {
-                                if (parsedTemp.getChild(j).getChildrenCount() == 0) {
-                                    stop = true;
-                                }
-                                else {
-                                    newParsedTemp = parsedTemp.getChild(j);
-                                }
-                            }
-                        }
-
-                        if (!stop && newParsedTemp != null) {
-                            parsedTemp = newParsedTemp;
                         }
                         else {
-                            stop = true;
+                            if (parsedTemp.getChild(j).getChildrenCount() == 0) {
+                                stop = true;
+                            }
+                            else {
+                                newParsedTemp = parsedTemp.getChild(j);
+                            }
                         }
                     }
+
+                    if (!stop && newParsedTemp != null) {
+                        parsedTemp = newParsedTemp;
+                    }
+                    else {
+                        stop = true;
+                    }
                 }
             }
         }
+
         // postprocess paths, now that all id vars are resolved
         if (paths != null) {
             for (EJBQLPath path : paths) {
@@ -156,34 +155,31 @@ class Compiler {
                         incoming = ((ArcProperty) property).getRelationship();
                         descriptor = ((ArcProperty) property).getTargetDescriptor();
                         pathRelationshipString = buffer.substring(0, buffer.length());
-                        
+
                         descriptorsById.put(pathRelationshipString, descriptor);
                         incomingById.put(pathRelationshipString, incoming);
-                        
+
                     }
                 }
 
-                
                 if (pathsInSelect.size() > 0
                         && incoming != null
                         && pathRelationshipString.length() > 0
                         && pathRelationshipString.equals(buffer.toString())) {
-                    
+
                     EJBQLIdentifier ident = new EJBQLIdentifier(0);
                     ident.text = pathRelationshipString;
-                    
+
                     Integer integer = pathsInSelect.get(path);
-                    if (integer!=null) {
+                    if (integer != null) {
                         resultComponents.remove(integer.intValue());
-                        resultComponents.add(pathsInSelect.get(path).intValue(), ident);
+                        resultComponents.add(integer, ident);
                         rootId = pathRelationshipString;
                     }
                 }
             }
         }
-       
-        
-        
+
         CompiledExpression compiled = new CompiledExpression();
         compiled.setExpression(parsed);
         compiled.setSource(source);
@@ -192,7 +188,7 @@ class Compiler {
         compiled.setDescriptorsById(descriptorsById);
         compiled.setIncomingById(incomingById);
         compiled.setPrefetchTree(prefetchTree);
-        
+
         if (resultComponents != null) {
             SQLResult mapping = new SQLResult();
 
@@ -222,26 +218,29 @@ class Compiler {
                         }
                     }
                     mapping.addEntityResult(compileEntityResult);
-                    
+
                 }
             }
 
             compiled.setResult(mapping);
-            
+
         }
 
         return compiled;
     }
 
-    private EntityResult compileEntityResultWithPrefetch(EntityResult compiledResult, EJBQLExpression prefetchExpression){
+    private EntityResult compileEntityResultWithPrefetch(
+            EntityResult compiledResult,
+            EJBQLExpression prefetchExpression) {
         final EntityResult result = compiledResult;
         String id = prefetchExpression.getText().toLowerCase();
         ClassDescriptor descriptor = descriptorsById.get(id);
         if (descriptor == null) {
             descriptor = descriptorsById.get(prefetchExpression.getText());
         }
-        final String prefix = prefetchExpression.getText().substring(prefetchExpression.getText().indexOf(".")+1);
-      
+        final String prefix = prefetchExpression.getText().substring(
+                prefetchExpression.getText().indexOf(".") + 1);
+
         final Set<String> visited = new HashSet<String>();
 
         PropertyVisitor visitor = new PropertyVisitor() {
@@ -249,10 +248,10 @@ class Compiler {
             public boolean visitAttribute(AttributeProperty property) {
                 ObjAttribute oa = property.getAttribute();
                 if (visited.add(oa.getDbAttributePath())) {
-                    result.addObjectField(
-                            oa.getEntity().getName(),
-                            "fetch."+prefix+"."+oa.getName(),
-                            prefix +"."+ oa.getDbAttributeName());
+                    result.addObjectField(oa.getEntity().getName(), "fetch."
+                            + prefix
+                            + "."
+                            + oa.getName(), prefix + "." + oa.getDbAttributeName());
                 }
                 return true;
             }
@@ -268,7 +267,9 @@ class Compiler {
                 for (DbJoin join : dbRel.getJoins()) {
                     DbAttribute src = join.getSource();
                     if (src.isForeignKey() && visited.add(src.getName())) {
-                        result.addDbField("fetch."+prefix+"."+src.getName(), prefix +"."+ src.getName());
+                        result.addDbField("fetch." + prefix + "." + src.getName(), prefix
+                                + "."
+                                + src.getName());
                     }
                 }
 
@@ -281,30 +282,35 @@ class Compiler {
         // append id columns ... (some may have been appended already via relationships)
         for (String pkName : descriptor.getEntity().getPrimaryKeyNames()) {
             if (visited.add(pkName)) {
-                result.addDbField("fetch."+prefix+"."+pkName, prefix +"."+ pkName);
+                result
+                        .addDbField("fetch." + prefix + "." + pkName, prefix
+                                + "."
+                                + pkName);
             }
         }
 
         // append inheritance discriminator columns...
-        Iterator<ObjAttribute> discriminatorColumns = descriptor
-                .getDiscriminatorColumns();
-        while (discriminatorColumns.hasNext()) {
-            ObjAttribute column = discriminatorColumns.next();
+        for (ObjAttribute column : descriptor.getDiscriminatorColumns()) {
 
             if (visited.add(column.getName())) {
-                result.addDbField("fetch."+prefix+"."+column.getDbAttributePath(), prefix +"."+ column.getDbAttributePath());
+                result.addDbField(
+                        "fetch." + prefix + "." + column.getDbAttributePath(),
+                        prefix + "." + column.getDbAttributePath());
             }
         }
 
         return result;
     }
-    
+
     private EntityResult compileEntityResult(EJBQLExpression expression, int position) {
         String id = expression.getText().toLowerCase();
         ClassDescriptor descriptor = descriptorsById.get(id);
         if (descriptor == null) {
             descriptor = descriptorsById.get(expression.getText());
         }
+        if(descriptor == null) {
+            throw new EJBQLException("the entity variable '" + id +"' does not refer to any entity in the FROM clause");
+        }
         final EntityResult entityResult = new EntityResult(descriptor.getObjectClass());
         final String prefix = "ec" + position + "_";
         final int[] index = {
@@ -355,10 +361,7 @@ class Compiler {
         }
 
         // append inheritance discriminator columns...
-        Iterator<ObjAttribute> discriminatorColumns = descriptor
-                .getDiscriminatorColumns();
-        while (discriminatorColumns.hasNext()) {
-            ObjAttribute column = discriminatorColumns.next();
+        for (ObjAttribute column : descriptor.getDiscriminatorColumns()) {
 
             if (visited.add(column.getName())) {
                 entityResult.addDbField(column.getDbAttributePath(), prefix + index[0]++);
@@ -458,7 +461,7 @@ class Compiler {
         public boolean visitSubselect(EJBQLExpression expression) {
             return super.visitSubselect(expression);
         }
-        
+
         private void prepareFetchJoin(EJBQLJoin join) {
             if (prefetchTree == null) {
                 prefetchTree = new PrefetchTreeNode();
@@ -530,7 +533,7 @@ class Compiler {
         @Override
         public boolean visitPath(EJBQLExpression expression, int finishedChildIndex) {
             if (finishedChildIndex + 1 < expression.getChildrenCount()) {
-                this.id = ((EJBQLPath) expression).getId();
+                this.id = normalizeIdPath(((EJBQLPath) expression).getId());
                 this.descriptor = descriptorsById.get(id);
 
                 if (descriptor == null) {
@@ -560,7 +563,7 @@ class Compiler {
         public boolean visitIdentifier(EJBQLExpression expression) {
             if (incoming != null) {
 
-                String aliasId = expression.getText();
+                String aliasId = expression.getText().toLowerCase();
 
                 // map id variable to class descriptor
                 ClassDescriptor old = descriptorsById.put(aliasId, descriptor);
@@ -609,7 +612,7 @@ class Compiler {
             expression.getChild(0).getChild(0).visit(pathVisitor);
             return false;
         }
-        
+
         @Override
         public boolean visitPath(EJBQLExpression expression, int finishedChildIndex) {
             addPath((EJBQLPath) expression);
