@@ -33,12 +33,4 @@ public class EJBQLNamedInputParameter extends SimpleNode {
     protected boolean visitNode(EJBQLExpressionVisitor visitor) {
         return visitor.visitNamedInputParameter(this);
     }
-   
-    /**
-     * @deprecated since 3.0.1 this method is no longer used
-     */
-    @Deprecated
-	public EJBQLEquals getParent() {
-        return (EJBQLEquals) parent;
-    }
 }
