@@ -32,6 +32,8 @@ import java.util.Map;
 
 import org.apache.cayenne.exp.parser.ASTScalar;
 import org.apache.cayenne.exp.parser.ExpressionParser;
+import org.apache.cayenne.exp.parser.ExpressionParserTokenManager;
+import org.apache.cayenne.exp.parser.JavaCharStream;
 import org.apache.cayenne.exp.parser.ParseException;
 import org.apache.cayenne.util.ConversionUtil;
 import org.apache.cayenne.util.Util;
@@ -80,9 +82,9 @@ public abstract class Expression implements Serializable, XMLSerializable {
      * <ul>
      * <li><i>An attribute of root ObjEntity.</i> For entity Gallery OBJ_PATH expression
      * "galleryName" will point to ObjAttribute "galleryName"
-     * <li><i>Another ObjEntity related to root ObjEntity via a chain of relationships.</i>
-     * For entity Gallery OBJ_PATH expression "paintingArray.toArtist" will point to
-     * ObjEntity "Artist"
+     * <li><i>Another ObjEntity related to root ObjEntity via a chain of
+     * relationships.</i> For entity Gallery OBJ_PATH expression "paintingArray.toArtist"
+     * will point to ObjEntity "Artist"
      * <li><i>ObjAttribute of another ObjEntity related to root ObjEntity via a chain of
      * relationships.</i> For entity Gallery OBJ_PATH expression
      * "paintingArray.toArtist.artistName" will point to ObjAttribute "artistName"
@@ -96,13 +98,13 @@ public abstract class Expression implements Serializable, XMLSerializable {
      * (dot). Path can point to either one of these:
      * <ul>
      * <li><i>An attribute of root DbEntity.</i> For entity GALLERY, DB_PATH expression
-     * "GALLERY_NAME" will point to a DbAttribute "GALLERY_NAME". </li>
+     * "GALLERY_NAME" will point to a DbAttribute "GALLERY_NAME".</li>
      * <li><i>Another DbEntity related to root DbEntity via a chain of relationships.</i>
      * For entity GALLERY DB_PATH expression "paintingArray.toArtist" will point to
-     * DbEntity "ARTIST". </li>
+     * DbEntity "ARTIST".</li>
      * <li><i>DbAttribute of another ObjEntity related to root DbEntity via a chain of
      * relationships.</i> For entity GALLERY DB_PATH expression
-     * "paintingArray.toArtist.ARTIST_NAME" will point to DbAttribute "ARTIST_NAME". </li>
+     * "paintingArray.toArtist.ARTIST_NAME" will point to DbAttribute "ARTIST_NAME".</li>
      * </ul>
      */
     public static final int DB_PATH = 27;
@@ -116,6 +118,28 @@ public abstract class Expression implements Serializable, XMLSerializable {
     public static final int NOT_IN = 36;
     public static final int NOT_LIKE = 37;
     public static final int NOT_LIKE_IGNORE_CASE = 38;
+    
+    /**
+     * @since 3.1
+     */
+    public static final int BITWISE_NOT = 39;
+    
+    /**
+     * @since 3.1
+     */
+    public static final int BITWISE_AND = 40;
+    
+    /**
+     * @since 3.1
+     */
+    public static final int BITWISE_OR = 41;
+    
+    /**
+     * @since 3.1
+     */
+    public static final int BITWISE_XOR = 42;
+
+    private static final int PARSE_BUFFER_MAX_SIZE = 4096;
 
     protected int type;
 
@@ -131,16 +155,32 @@ public abstract class Expression implements Serializable, XMLSerializable {
             throw new NullPointerException("Null expression string.");
         }
 
+        // optimizing parser buffers per CAY-1667...
+        // adding 1 extra char to the buffer size above the String length, as otherwise
+        // resizing still occurs at the end of the stream
+        int bufferSize = expressionString.length() > PARSE_BUFFER_MAX_SIZE
+                ? PARSE_BUFFER_MAX_SIZE
+                : expressionString.length() + 1;
         Reader reader = new StringReader(expressionString);
+        JavaCharStream stream = new JavaCharStream(reader, 1, 1, bufferSize);
+        ExpressionParserTokenManager tm = new ExpressionParserTokenManager(stream);
+        ExpressionParser parser = new ExpressionParser(tm);
+
         try {
-            return new ExpressionParser(reader).expression();
+            return parser.expression();
         }
         catch (ParseException ex) {
-            throw new ExpressionException(ex.getMessage(), ex);
+
+            // can be null
+            String message = ex.getMessage();
+            throw new ExpressionException(message != null ? message : "", ex);
         }
         catch (Throwable th) {
+            // can be null
+            String message = th.getMessage();
+            
             // another common error is TokenManagerError
-            throw new ExpressionException(th.getMessage(), th);
+            throw new ExpressionException(message != null ? message : "", th);
         }
     }
 
@@ -258,8 +298,8 @@ public abstract class Expression implements Serializable, XMLSerializable {
      *            expression parameter, and value being the value that should be used in
      *            the final expression.
      * @param pruneMissing If <code>true</code>, subexpressions that rely on missing
-     *            parameters will be pruned from the resulting tree. If <code>false</code>,
-     *            any missing values will generate an exception.
+     *            parameters will be pruned from the resulting tree. If <code>false</code>
+     *            , any missing values will generate an exception.
      * @return Expression resulting from the substitution of parameters with real values,
      *         or null if the whole expression was pruned, due to the missing parameters.
      */
@@ -272,6 +312,21 @@ public abstract class Expression implements Serializable, XMLSerializable {
 
             public Object transform(Object object) {
                 if (!(object instanceof ExpressionParameter)) {
+
+                    // mainly for the ASTList array child...
+                    if (object instanceof Object[]) {
+
+                        Object[] source = (Object[]) object;
+                        int len = source.length;
+                        Object[] target = new Object[len];
+
+                        for (int i = 0; i < len; i++) {
+                            target[i] = transform(source[i]);
+                        }
+                        
+                        return target;
+                    }
+
                     return object;
                 }
 
@@ -336,8 +391,8 @@ public abstract class Expression implements Serializable, XMLSerializable {
     public abstract Expression notExp();
 
     /**
-     * Returns a count of operands of this expression. In real life there are unary (count ==
-     * 1), binary (count == 2) and ternary (count == 3) expressions.
+     * Returns a count of operands of this expression. In real life there are unary (count
+     * == 1), binary (count == 2) and ternary (count == 3) expressions.
      */
     public abstract int getOperandCount();
 
@@ -366,6 +421,21 @@ public abstract class Expression implements Serializable, XMLSerializable {
     public boolean match(Object o) {
         return ConversionUtil.toBoolean(evaluate(o));
     }
+    
+    /**
+     * Returns the first object in the list that matches the expression.
+     * 
+     * @since 3.1
+     */
+    public <T> T first(List<T> objects) {
+        for (T o : objects) {
+            if (match(o)) {
+                return o;
+            }
+        }
+
+        return null;
+    }
 
     /**
      * Returns a list of objects that match the expression.
