@@ -31,6 +31,8 @@ import java.util.SortedMap;
 
 import org.apache.cayenne.CayenneRuntimeException;
 import org.apache.cayenne.ObjectId;
+import org.apache.cayenne.configuration.ConfigurationNode;
+import org.apache.cayenne.configuration.ConfigurationNodeVisitor;
 import org.apache.cayenne.exp.Expression;
 import org.apache.cayenne.exp.ExpressionException;
 import org.apache.cayenne.exp.ExpressionFactory;
@@ -49,7 +51,7 @@ import org.apache.commons.collections.Transformer;
 /**
  * A DbEntity is a mapping descriptor that defines a structure of a database table.
  */
-public class DbEntity extends Entity implements DbEntityListener, DbAttributeListener,
+public class DbEntity extends Entity implements ConfigurationNode, DbEntityListener, DbAttributeListener,
         DbRelationshipListener {
 
     protected String catalog;
@@ -84,6 +86,13 @@ public class DbEntity extends Entity implements DbEntityListener, DbAttributeLis
         this();
         this.setName(name);
     }
+    
+    /**
+     * @since 3.1
+     */
+    public <T> T acceptVisitor(ConfigurationNodeVisitor<T> visitor) {
+        return visitor.visitDbEntity(this);
+    }
 
     /**
      * Prints itself as XML to the provided XMLEncoder.
@@ -173,14 +182,6 @@ public class DbEntity extends Entity implements DbEntityListener, DbAttributeLis
         return Collections.unmodifiableCollection(primaryKey);
     }
 
-    /**
-     * @deprecated since 3.0 use {@link #getPrimaryKeys()} that returns a collection.
-     */
-    @Deprecated
-    public List<DbAttribute> getPrimaryKey() {
-        return new ArrayList<DbAttribute>(getPrimaryKeys());
-    }
-
     /**
      * Returns a Collection of all attributes that either belong to this DbEntity or
      * inherited.
@@ -214,19 +215,6 @@ public class DbEntity extends Entity implements DbEntityListener, DbAttributeLis
         this.dbAttributeAdded(new AttributeEvent(this, attr, this, MapEvent.ADD));
     }
 
-    /**
-     * Overrides super to fire an AttributeEvent.
-     * 
-     * @deprecated in favour of {@link #addAttribute(DbAttribute attr)}. Scheduled for
-     *             removal in Cayenne 4.
-     */
-    @Override
-    @Deprecated
-    public void addAttribute(Attribute attr) {
-        super.addAttribute(attr);
-        this.dbAttributeAdded(new AttributeEvent(this, attr, this, MapEvent.ADD));
-    }
-
     /**
      * Removes attribute from the entity, removes any relationship joins containing this
      * attribute. Does nothing if the attribute name is not found.
@@ -537,7 +525,19 @@ public class DbEntity extends Entity implements DbEntityListener, DbAttributeLis
         // handle relationship name changes
         if (e.getId() == RelationshipEvent.CHANGE && e.isNameChange()) {
             String oldName = e.getOldName();
-
+            
+            DataMap map = getDataMap();
+            if (map != null) {
+                
+                // updating dbAttributePaths for attributes of all ObjEntities
+                for (ObjEntity objEntity : getDataMap().getObjEntities()) {
+                    
+                    for (ObjAttribute attribute : objEntity.getAttributes()) {
+                        attribute.updateDbAttributePath();
+                    }
+                }
+            }
+            
             // clear the relationship out of the collection
             relationships.remove(oldName);
 
@@ -809,9 +809,9 @@ public class DbEntity extends Entity implements DbEntityListener, DbAttributeLis
         private void appendPath(
                 LinkedList<String> finalPath,
                 PathComponent<Attribute, Relationship> pathComponent) {
-        	CayenneMapEntry mapEntry = pathComponent.getAttribute() != null ? 
-                    pathComponent.getAttribute() :
-                    pathComponent.getRelationship();
+            CayenneMapEntry mapEntry = pathComponent.getAttribute() != null ?
+                pathComponent.getAttribute() :
+                pathComponent.getRelationship();
             String name = mapEntry.getName();
             if (pathComponent.getJoinType() == JoinType.LEFT_OUTER) {
                 name += OUTER_JOIN_INDICATOR;
