@@ -19,15 +19,6 @@
 
 package org.apache.cayenne.map;
 
-import java.io.IOException;
-import java.io.ObjectInputStream;
-import java.io.Serializable;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.Map;
-
 import org.apache.cayenne.CayenneRuntimeException;
 import org.apache.cayenne.ObjectId;
 import org.apache.cayenne.Persistent;
@@ -38,13 +29,22 @@ import org.apache.cayenne.reflect.FaultFactory;
 import org.apache.cayenne.reflect.LifecycleCallbackRegistry;
 import org.apache.cayenne.reflect.SingletonFaultFactory;
 import org.apache.cayenne.reflect.generic.DataObjectDescriptorFactory;
-import org.apache.cayenne.reflect.pojo.EnhancedPojoDescriptorFactory;
 import org.apache.cayenne.reflect.valueholder.ValueHolderDescriptorFactory;
 import org.apache.cayenne.util.Util;
 import org.apache.commons.collections.collection.CompositeCollection;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.io.Serializable;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.Map;
+import java.util.concurrent.atomic.AtomicLong;
+
 /**
  * Represents a virtual shared namespace for zero or more DataMaps. Unlike DataMap,
  * EntityResolver is intended to work as a runtime container of mapping. DataMaps can be
@@ -60,6 +60,7 @@ public class EntityResolver implements MappingNamespace, Serializable {
     static final ObjEntity DUPLICATE_MARKER = new ObjEntity();
 
     protected static final Log logger = LogFactory.getLog(EntityResolver.class);
+    protected static AtomicLong incrementer = new AtomicLong();
 
     protected boolean indexedByClass;
 
@@ -138,7 +139,7 @@ public class EntityResolver implements MappingNamespace, Serializable {
                         DbRelationship reverse = relationship.createReverseRelationship();
 
                         Entity targetEntity = reverse.getSourceEntity();
-                        reverse.setName(makeUniqueRelationshipName(targetEntity));
+                        reverse.setName(getUniqueRelationshipName(targetEntity));
                         reverse.setRuntime(true);
                         targetEntity.addRelationship(reverse);
 
@@ -177,7 +178,7 @@ public class EntityResolver implements MappingNamespace, Serializable {
                                 .createReverseRelationship();
 
                         Entity targetEntity = reverse.getSourceEntity();
-                        reverse.setName(makeUniqueRelationshipName(targetEntity));
+                        reverse.setName(getUniqueRelationshipName(targetEntity));
                         reverse.setRuntime(true);
                         targetEntity.addRelationship(reverse);
 
@@ -191,16 +192,14 @@ public class EntityResolver implements MappingNamespace, Serializable {
         }
     }
 
-    private String makeUniqueRelationshipName(Entity entity) {
-        for (int i = 0; i < 1000; i++) {
-            String name = "runtimeRelationship" + i;
-            if (entity.getRelationship(name) == null) {
-                return name;
-            }
-        }
+    private String getUniqueRelationshipName(Entity entity) {
+        String name;
 
-        throw new CayenneRuntimeException(
-                "Could not come up with a unique relationship name");
+        do {
+            name = "runtimeRelationship" + incrementer.getAndIncrement();
+        } while(entity.getRelationship(name) != null);
+
+        return name;
     }
 
     /**
@@ -561,15 +560,15 @@ public class EntityResolver implements MappingNamespace, Serializable {
                     }
                 }
             }
-            
-            // index embeddables
-            embeddableCache.putAll(map.getEmbeddableMap());
 
             // index stored procedures
             for (Procedure proc : map.getProcedures()) {
                 procedureCache.put(proc.getName(), proc);
             }
 
+            // index embeddables
+            embeddableCache.putAll(map.getEmbeddableMap());
+
             // index queries
             for (Query query : map.getQueries()) {
                 String name = query.getName();
@@ -588,8 +587,7 @@ public class EntityResolver implements MappingNamespace, Serializable {
             // index ObjEntity inheritance
             for (ObjEntity oe : map.getObjEntities()) {
 
-                // build inheritance tree... include nodes that
-                // have no children to avoid unneeded cache rebuilding on lookup...
+                // build inheritance tree
                 EntityInheritanceTree node = entityInheritanceCache.get(oe.getName());
                 if (node == null) {
                     node = new EntityInheritanceTree(oe);
@@ -653,44 +651,21 @@ public class EntityResolver implements MappingNamespace, Serializable {
         return Collections.unmodifiableCollection(maps);
     }
 
-    /**
-     * Looks in the DataMap's that this object was created with for the DbEntity that
-     * services the specified class
-     * 
-     * @return the required DbEntity, or null if none matches the specifier
-     * @deprecated since 3.0 - lookup DbEntity via ObjEntity instead.
-     */
-    @Deprecated
-    public synchronized DbEntity lookupDbEntity(Class<?> aClass) {
-        ObjEntity oe = lookupObjEntity(aClass);
-        return oe != null ? oe.getDbEntity() : null;
-    }
-
-    /**
-     * Looks in the DataMap's that this object was created with for the DbEntity that
-     * services the specified data Object
-     * 
-     * @return the required DbEntity, or null if none matches the specifier
-     * @deprecated since 3.0 - lookup DbEntity via ObjEntity instead.
-     */
-    @Deprecated
-    public synchronized DbEntity lookupDbEntity(Persistent dataObject) {
-        return lookupDbEntity(dataObject.getClass());
-    }
-
     /**
      * Returns EntityInheritanceTree representing inheritance hierarchy that starts with a
-     * given ObjEntity as root, and includes all its subentities. If ObjEntity has no
-     * known subentities, null is returned.
+     * given ObjEntity as root, and includes all its subentities.
+     * 
+     * @deprecated since 3.1 use {@link #lookupInheritanceTree(String)}.
      */
+    @Deprecated
     public EntityInheritanceTree lookupInheritanceTree(ObjEntity entity) {
         return lookupInheritanceTree(entity.getName());
     }
 
     /**
      * Returns EntityInheritanceTree representing inheritance hierarchy that starts with a
-     * given ObjEntity as root, and includes all its subentities. If ObjEntity has no
-     * known subentities, null is returned.
+     * given ObjEntity as root, and includes all its subentities. Returns non-null object
+     * for all existing entities, even those that don't have super or subclasses.
      * 
      * @since 3.0
      */
@@ -708,8 +683,7 @@ public class EntityResolver implements MappingNamespace, Serializable {
             tree = entityInheritanceCache.get(entityName);
         }
 
-        // don't return "trivial" trees
-        return (tree == null || tree.getChildrenCount() == 0) ? null : tree;
+        return tree;
     }
 
     /**
@@ -750,18 +724,6 @@ public class EntityResolver implements MappingNamespace, Serializable {
         return lookupObjEntity(object.getClass());
     }
 
-    /**
-     * Looks in the DataMap's that this object was created with for the ObjEntity that
-     * maps to the services the class with the given name
-     * 
-     * @return the required ObjEntity or null if there is none that matches the specifier
-     * @deprecated since 3.0 - use getObjEntity() instead.
-     */
-    @Deprecated
-    public synchronized ObjEntity lookupObjEntity(String entityName) {
-        return _lookupObjEntity(entityName);
-    }
-
     public Procedure lookupProcedure(Query q) {
         return q.getMetaData(this).getProcedure();
     }
@@ -892,9 +854,6 @@ public class EntityResolver implements MappingNamespace, Serializable {
             // add factories in reverse of the desired chain order
             classDescriptorMap.addFactory(new ValueHolderDescriptorFactory(
                     classDescriptorMap));
-            classDescriptorMap.addFactory(new EnhancedPojoDescriptorFactory(
-                    classDescriptorMap,
-                    faultFactory));
             classDescriptorMap.addFactory(new DataObjectDescriptorFactory(
                     classDescriptorMap,
                     faultFactory));
