@@ -31,14 +31,14 @@ import java.util.SortedMap;
 import java.util.TreeMap;
 
 import org.apache.cayenne.CayenneRuntimeException;
+import org.apache.cayenne.configuration.ConfigurationNode;
+import org.apache.cayenne.configuration.ConfigurationNodeVisitor;
 import org.apache.cayenne.dba.TypesMapping;
 import org.apache.cayenne.exp.Expression;
 import org.apache.cayenne.exp.ExpressionException;
 import org.apache.cayenne.exp.ExpressionFactory;
-import org.apache.cayenne.map.event.AttributeEvent;
 import org.apache.cayenne.map.event.EntityEvent;
 import org.apache.cayenne.map.event.ObjEntityListener;
-import org.apache.cayenne.map.event.RelationshipEvent;
 import org.apache.cayenne.util.CayenneMapEntry;
 import org.apache.cayenne.util.NameConverter;
 import org.apache.cayenne.util.Util;
@@ -49,7 +49,7 @@ import org.apache.commons.collections.Transformer;
  * ObjEntity is a mapping descriptor for a DataObject Java class. It contains the
  * information about the Java class itself, as well as its mapping to the DbEntity layer.
  */
-public class ObjEntity extends Entity implements ObjEntityListener {
+public class ObjEntity extends Entity implements ObjEntityListener, ConfigurationNode {
 
     final public static int LOCK_TYPE_NONE = 0;
     final public static int LOCK_TYPE_OPTIMISTIC = 1;
@@ -95,6 +95,13 @@ public class ObjEntity extends Entity implements ObjEntityListener {
         this.entityListeners = new ArrayList<EntityListener>(2);
         this.attributeOverrides = new TreeMap<String, String>();
     }
+    
+    /**
+     * @since 3.1
+     */
+    public <T> T acceptVisitor(ConfigurationNodeVisitor<T> visitor) {
+        return visitor.visitObjEntity(this);
+    }
 
     /**
      * Prints itself as XML to the provided XMLEncoder.
@@ -138,8 +145,14 @@ public class ObjEntity extends Entity implements ObjEntityListener {
         }
 
         if (getDbEntityName() != null && getDbEntity() != null) {
-            encoder.print("\" dbEntityName=\"");
-            encoder.print(Util.encodeXmlAttribute(getDbEntityName()));
+            
+            // not writing DbEntity name if sub entity has same DbEntity 
+            // as super entity, see CAY-1477
+            if (!(getSuperEntity() != null 
+                    && getSuperEntity().getDbEntity() == getDbEntity())) {
+                encoder.print("\" dbEntityName=\"");
+                encoder.print(Util.encodeXmlAttribute(getDbEntityName()));
+            }
         }
 
         if (getSuperEntityName() == null && getSuperClassName() != null) {
@@ -1116,7 +1129,6 @@ public class ObjEntity extends Entity implements ObjEntityListener {
                             + component);
                 }
 
-                boolean firstComponent = true;
                 while (dbSubpath.hasNext()) {
                     CayenneMapEntry subComponent = (CayenneMapEntry) dbSubpath.next();
                     if (buf.length() > 0) {
@@ -1124,10 +1136,11 @@ public class ObjEntity extends Entity implements ObjEntityListener {
                     }
 
                     buf.append(subComponent.getName());
-                    if (firstComponent && component.getJoinType() == JoinType.LEFT_OUTER) {
+                    
+                    // use OUTER join for all components of the path is Obj path is OUTER
+                    if (component.getJoinType() == JoinType.LEFT_OUTER) {
                         buf.append(OUTER_JOIN_INDICATOR);
                     }
-                    firstComponent = false;
                 }
             }
 
@@ -1216,66 +1229,6 @@ public class ObjEntity extends Entity implements ObjEntityListener {
         // does nothing currently
     }
 
-    /**
-     * Attribute property changed.
-     * 
-     * @deprecated since 3.0 ObjEntity no longer implements ObjAttributeListener
-     */
-    @Deprecated
-    public void objAttributeChanged(AttributeEvent e) {
-        // does nothing currently
-    }
-
-    /**
-     * New attribute has been created/added.
-     * 
-     * @deprecated since 3.0 ObjEntity no longer implements ObjAttributeListener
-     */
-    @Deprecated
-    public void objAttributeAdded(AttributeEvent e) {
-        // does nothing currently
-    }
-
-    /**
-     * Attribute has been removed.
-     * 
-     * @deprecated since 3.0 ObjEntity no longer implements ObjAttributeListener
-     */
-    @Deprecated
-    public void objAttributeRemoved(AttributeEvent e) {
-        // does nothing currently
-    }
-
-    /**
-     * Relationship property changed.
-     * 
-     * @deprecated since 3.0 ObjEntity no longer implements ObjRelationshipListener
-     */
-    @Deprecated
-    public void objRelationshipChanged(RelationshipEvent e) {
-        // does nothing currently
-    }
-
-    /**
-     * Relationship has been created/added.
-     * 
-     * @deprecated since 3.0 ObjEntity no longer implements ObjRelationshipListener
-     */
-    @Deprecated
-    public void objRelationshipAdded(RelationshipEvent e) {
-        // does nothing currently
-    }
-
-    /**
-     * Relationship has been removed.
-     * 
-     * @deprecated since 3.0 ObjEntity no longer implements ObjRelationshipListener
-     */
-    @Deprecated
-    public void objRelationshipRemoved(RelationshipEvent e) {
-        // does nothing currently
-    }
-
     /**
      * Returns true if the default lifecycle listeners should not be notified of this
      * entity lifecycle events.
@@ -1303,5 +1256,4 @@ public class ObjEntity extends Entity implements ObjEntityListener {
     public void setExcludingSuperclassListeners(boolean excludingSuperclassListeners) {
         this.excludingSuperclassListeners = excludingSuperclassListeners;
     }
-
 }
