@@ -20,39 +20,59 @@
 package org.apache.cayenne.access;
 
 import java.util.Collection;
-import java.util.Collections;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
-import org.apache.art.Artist;
-import org.apache.art.Painting;
 import org.apache.cayenne.DataRow;
 import org.apache.cayenne.ObjectId;
 import org.apache.cayenne.PersistenceState;
+import org.apache.cayenne.di.Inject;
 import org.apache.cayenne.exp.Expression;
 import org.apache.cayenne.exp.ExpressionFactory;
-import org.apache.cayenne.query.ObjectIdQuery;
 import org.apache.cayenne.query.SQLTemplate;
 import org.apache.cayenne.query.SelectQuery;
-import org.apache.cayenne.unit.MultiContextCase;
+import org.apache.cayenne.test.jdbc.DBHelper;
+import org.apache.cayenne.testdo.testmap.Artist;
+import org.apache.cayenne.testdo.testmap.Painting;
+import org.apache.cayenne.unit.di.server.ServerCase;
+import org.apache.cayenne.unit.di.server.UseServerRuntime;
+import org.apache.cayenne.unit.util.SQLTemplateCustomizer;
 import org.apache.cayenne.unit.util.ThreadedTestHelper;
 
 /**
  * Test suite for testing behavior of multiple DataContexts that share the same underlying
  * DataDomain.
  */
-public class DataContextSharedCacheTest extends MultiContextCase {
+@UseServerRuntime(ServerCase.TESTMAP_PROJECT)
+public class DataContextSharedCacheTest extends ServerCase {
 
-    protected Artist artist;
-    protected DataContext context;
+    @Inject
+    private DataContext context;
+
+    @Inject
+    protected DBHelper dbHelper;
+
+    @Inject
+    private DataContext context1;
+
+    @Inject
+    private SQLTemplateCustomizer sqlTemplateCustomizer;
+
+    private Artist artist;
 
     @Override
-    protected void setUp() throws Exception {
-        super.setUp();
+    protected void setUpAfterInjection() throws Exception {
 
-        context = createDataContextWithSharedCache();
+    	dbHelper.deleteAll("PAINTING_INFO");
+        dbHelper.deleteAll("PAINTING");
+        dbHelper.deleteAll("PAINTING1");
+        dbHelper.deleteAll("ARTIST_EXHIBIT");
+        dbHelper.deleteAll("ARTIST_GROUP");
+        dbHelper.deleteAll("ARTIST");
+        dbHelper.deleteAll("EXHIBIT");
+        dbHelper.deleteAll("GALLERY");
 
         // prepare a single artist record
         artist = (Artist) context.newObject("Artist");
@@ -63,17 +83,14 @@ public class DataContextSharedCacheTest extends MultiContextCase {
 
     /**
      * Test case to prove that refreshing snapshots as a result of the database fetch will
-     * be propagated accross DataContexts.
+     * be propagated across DataContexts.
      */
     public void testSnapshotChangePropagationOnSelect() throws Exception {
         String originalName = artist.getArtistName();
         final String newName = "version2";
 
-        // create alternative context making sure that no cache is flushed
-        DataContext altContext = context.getParentDataDomain().createDataContext(true);
-
         // update artist using raw SQL
-        SQLTemplate query = getSQLTemplateBuilder().createSQLTemplate(
+        SQLTemplate query = sqlTemplateCustomizer.createSQLTemplate(
                 Artist.class,
                 "UPDATE ARTIST SET ARTIST_NAME = #bind($newName) "
                         + "WHERE ARTIST_NAME = #bind($oldName)");
@@ -87,7 +104,7 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         // fetch updated artist into the new context, and see if the original
         // one gets updated
         Expression qual = ExpressionFactory.matchExp("artistName", newName);
-        List artists = altContext.performQuery(new SelectQuery(Artist.class, qual));
+        List artists = context1.performQuery(new SelectQuery(Artist.class, qual));
         assertEquals(1, artists.size());
         Artist altArtist = (Artist) artists.get(0);
 
@@ -119,20 +136,14 @@ public class DataContextSharedCacheTest extends MultiContextCase {
      * Test case to prove that changes made to an object in one ObjectStore and committed
      * to the database will be reflected in the peer ObjectStore using the same
      * DataRowCache.
-     * 
-     * @throws Exception
      */
     public void testSnapshotChangePropagation() throws Exception {
         String originalName = artist.getArtistName();
         final String newName = "version2";
 
-        DataContext altContext = mirrorDataContext(context);
-
         // make sure we have a fully resolved copy of an artist object
         // in the second context
-        final Artist altArtist = (Artist) altContext.getGraphManager().getNode(
-                artist.getObjectId());
-        assertNotNull(altArtist);
+        final Artist altArtist = context1.localObject(artist);
         assertFalse(altArtist == artist);
         assertEquals(originalName, altArtist.getArtistName());
         assertEquals(PersistenceState.COMMITTED, altArtist.getPersistenceState());
@@ -176,12 +187,9 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         final Date newDate = new Date(originalDate.getTime() - 10000);
         final String newAltName = "version3";
 
-        DataContext altContext = mirrorDataContext(context);
-
         // make sure we have a fully resolved copy of an artist object
         // in the second context
-        final Artist altArtist = (Artist) altContext.getGraphManager().getNode(
-                artist.getObjectId());
+        final Artist altArtist = context1.localObject(artist);
         assertNotNull(altArtist);
         assertFalse(altArtist == artist);
         assertEquals(originalName, altArtist.getArtistName());
@@ -220,29 +228,26 @@ public class DataContextSharedCacheTest extends MultiContextCase {
      * Test case to prove that deleting an object in one ObjectStore and committing to the
      * database will be reflected in the peer ObjectStore using the same DataRowCache. By
      * default COMMITTED objects will be changed to TRANSIENT.
-     * 
-     * @throws Exception
      */
     public void testSnapshotDeletePropagationToCommitted() throws Exception {
 
-        DataContext altContext = mirrorDataContext(context);
-
         // make sure we have a fully resolved copy of an artist object
         // in the second context
-        final Artist altArtist = (Artist) altContext.getGraphManager().getNode(
-                artist.getObjectId());
+        final Artist altArtist = context1.localObject(artist);
         assertNotNull(altArtist);
         assertFalse(altArtist == artist);
         assertEquals(artist.getArtistName(), altArtist.getArtistName());
         assertEquals(PersistenceState.COMMITTED, altArtist.getPersistenceState());
 
         // Update Artist
-        context.deleteObject(artist);
+        context.deleteObjects(artist);
         context.commitChanges();
 
         // check underlying cache
-        assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(
-                altArtist.getObjectId()));
+        assertNull(context
+                .getObjectStore()
+                .getDataRowCache()
+                .getCachedSnapshot(altArtist.getObjectId()));
 
         // check peer artist
         ThreadedTestHelper helper = new ThreadedTestHelper() {
@@ -265,23 +270,21 @@ public class DataContextSharedCacheTest extends MultiContextCase {
      */
     public void testSnapshotDeletePropagationToHollow() throws Exception {
 
-        DataContext altContext = mirrorDataContext(context);
+        final Artist altArtist = context1.localObject(artist);
 
-        // make sure we have a fully resolved copy of an artist object
-        // in the second context
-        final Artist altArtist = (Artist) altContext.getGraphManager().getNode(
-                artist.getObjectId());
         assertNotNull(altArtist);
         assertFalse(altArtist == artist);
         assertEquals(PersistenceState.HOLLOW, altArtist.getPersistenceState());
 
         // Update Artist
-        context.deleteObject(artist);
+        context.deleteObjects(artist);
         context.commitChanges();
 
         // check underlying cache
-        assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(
-                altArtist.getObjectId()));
+        assertNull(context
+                .getObjectStore()
+                .getDataRowCache()
+                .getCachedSnapshot(altArtist.getObjectId()));
 
         // check peer artist
         ThreadedTestHelper helper = new ThreadedTestHelper() {
@@ -302,12 +305,10 @@ public class DataContextSharedCacheTest extends MultiContextCase {
      */
     public void testSnapshotDeletePropagationToModified() throws Exception {
 
-        DataContext altContext = mirrorDataContext(context);
-
         // make sure we have a fully resolved copy of an artist object
         // in the second context
-        final Artist altArtist = (Artist) altContext.getGraphManager().getNode(
-                artist.getObjectId());
+        final Artist altArtist = context1.localObject(artist);
+        altArtist.getArtistName();
         assertNotNull(altArtist);
         assertFalse(altArtist == artist);
 
@@ -316,12 +317,14 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         assertEquals(PersistenceState.MODIFIED, altArtist.getPersistenceState());
 
         // Update Artist
-        context.deleteObject(artist);
+        context.deleteObjects(artist);
         context.commitChanges();
 
         // check underlying cache
-        assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(
-                altArtist.getObjectId()));
+        assertNull(context
+                .getObjectStore()
+                .getDataRowCache()
+                .getCachedSnapshot(altArtist.getObjectId()));
 
         // check peer artist
         ThreadedTestHelper helper = new ThreadedTestHelper() {
@@ -340,16 +343,9 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         assertNotNull(id.getIdSnapshot().get(Artist.ARTIST_ID_PK_COLUMN));
         assertFalse(id.isTemporary());
 
-        altContext.commitChanges();
+        context1.commitChanges();
 
         assertEquals(PersistenceState.COMMITTED, altArtist.getPersistenceState());
-
-        // create independent context and fetch artist in it
-        DataContext context3 = getDomain().createDataContext(false);
-        List artists = context3.performQuery(new ObjectIdQuery(id));
-        assertEquals(1, artists.size());
-        Artist artist3 = (Artist) artists.get(0);
-        assertEquals(id, artist3.getObjectId());
     }
 
     /**
@@ -361,25 +357,25 @@ public class DataContextSharedCacheTest extends MultiContextCase {
      */
     public void testSnapshotDeletePropagationToDeleted() throws Exception {
 
-        DataContext altContext = mirrorDataContext(context);
-
         // make sure we have a fully resolved copy of an artist object
         // in the second context
-        final Artist altArtist = (Artist) altContext.getGraphManager().getNode(
-                artist.getObjectId());
+        final Artist altArtist = context1.localObject(artist);
+        altArtist.getArtistName();
         assertNotNull(altArtist);
         assertFalse(altArtist == artist);
 
         // delete peer
-        altContext.deleteObject(altArtist);
+        context1.deleteObjects(altArtist);
 
         // Update Artist
-        context.deleteObject(artist);
+        context.deleteObjects(artist);
         context.commitChanges();
 
         // check underlying cache
-        assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(
-                altArtist.getObjectId()));
+        assertNull(context
+                .getObjectStore()
+                .getDataRowCache()
+                .getCachedSnapshot(altArtist.getObjectId()));
 
         // check peer artist
         ThreadedTestHelper helper = new ThreadedTestHelper() {
@@ -392,15 +388,13 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         };
         helper.assertWithTimeout(3000);
 
-        assertFalse(altContext.hasChanges());
+        assertFalse(context1.hasChanges());
     }
 
     /**
      * Test case to prove that deleting an object in one ObjectStore and committing to the
      * database will be reflected in the peer ObjectStore using the same DataRowCache,
      * including proper processing of deleted object being held in to-many collections.
-     * 
-     * @throws Exception
      */
     public void testSnapshotDeletePropagationToManyRefresh() throws Exception {
 
@@ -414,18 +408,13 @@ public class DataContextSharedCacheTest extends MultiContextCase {
 
         context.commitChanges();
 
-        DataContext altContext = mirrorDataContext(context);
-
         // make sure we have a fully resolved copy of an artist and painting
         // objects
         // in the second context
 
-        final Artist altArtist = (Artist) altContext.getGraphManager().getNode(
-                artist.getObjectId());
-        final Painting altPainting1 = (Painting) altContext.getGraphManager().getNode(
-                painting1.getObjectId());
-        final Painting altPainting2 = (Painting) altContext.getGraphManager().getNode(
-                painting2.getObjectId());
+        final Artist altArtist = context1.localObject(artist);
+        final Painting altPainting1 = context1.localObject(painting1);
+        final Painting altPainting2 = context1.localObject(painting2);
 
         assertEquals(artist.getArtistName(), altArtist.getArtistName());
         assertEquals(painting1.getPaintingTitle(), altPainting1.getPaintingTitle());
@@ -443,23 +432,26 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         assertSame(altArtist, altPainting2.readPropertyDirectly("toArtist"));
 
         // delete painting
-        context.deleteObject(painting1);
+        context.deleteObjects(painting1);
         context.commitChanges();
 
         // check underlying cache
-        assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(
-                painting1.getObjectId()));
+        assertNull(context
+                .getObjectStore()
+                .getDataRowCache()
+                .getCachedSnapshot(painting1.getObjectId()));
 
         // check peer artist
         ThreadedTestHelper helper = new ThreadedTestHelper() {
 
             @Override
             protected void assertResult() throws Exception {
-                assertEquals(PersistenceState.TRANSIENT, altPainting1
-                        .getPersistenceState());
+                assertEquals(
+                        PersistenceState.TRANSIENT,
+                        altPainting1.getPersistenceState());
                 assertEquals(PersistenceState.COMMITTED, altArtist.getPersistenceState());
 
-                Collection list = altArtist.getPaintingArray();
+                Collection<Painting> list = altArtist.getPaintingArray();
                 assertEquals(1, list.size());
                 assertFalse(list.contains(altPainting1));
             }
@@ -482,16 +474,12 @@ public class DataContextSharedCacheTest extends MultiContextCase {
 
         context.commitChanges();
 
-        DataContext altContext = mirrorDataContext(context);
-
         // make sure we have a fully resolved copy of an artist and painting
         // objects
         // in the second context
 
-        final Artist altArtist = (Artist) altContext.getGraphManager().getNode(
-                artist.getObjectId());
-        final Painting altPainting1 = (Painting) altContext.getGraphManager().getNode(
-                painting1.getObjectId());
+        final Artist altArtist = context1.localObject(artist);
+        final Painting altPainting1 = context1.localObject(painting1);
 
         assertEquals(artist.getArtistName(), altArtist.getArtistName());
         assertEquals(painting1.getPaintingTitle(), altPainting1.getPaintingTitle());
@@ -519,15 +507,13 @@ public class DataContextSharedCacheTest extends MultiContextCase {
             }
         };
         helper.assertWithTimeout(2000);
-        List list = altArtist.getPaintingArray();
+        List<Painting> list = altArtist.getPaintingArray();
         assertEquals(2, list.size());
     }
 
     /**
      * Checks that cache is refreshed when a query "refreshingObjects" property is set to
      * true.
-     * 
-     * @throws Exception
      */
     public void testCacheRefreshingOnSelect() throws Exception {
         String originalName = artist.getArtistName();
@@ -543,7 +529,7 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         assertEquals(originalName, oldSnapshot.get("ARTIST_NAME"));
 
         // update artist using raw SQL
-        SQLTemplate update = getSQLTemplateBuilder()
+        SQLTemplate update = sqlTemplateCustomizer
                 .createSQLTemplate(
                         Artist.class,
                         "UPDATE ARTIST SET ARTIST_NAME = #bind($newName) WHERE ARTIST_NAME = #bind($oldName)");
@@ -575,10 +561,12 @@ public class DataContextSharedCacheTest extends MultiContextCase {
     public void testSnapshotEvictedForHollow() throws Exception {
         String originalName = artist.getArtistName();
 
-        context.invalidateObjects(Collections.singletonList(artist));
+        context.invalidateObjects(artist);
         assertEquals(PersistenceState.HOLLOW, artist.getPersistenceState());
-        assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(
-                artist.getObjectId()));
+        assertNull(context
+                .getObjectStore()
+                .getDataRowCache()
+                .getCachedSnapshot(artist.getObjectId()));
 
         // resolve object
         assertEquals(originalName, artist.getArtistName());
@@ -593,22 +581,20 @@ public class DataContextSharedCacheTest extends MultiContextCase {
     public void testSnapshotEvictedAndObjectsHollowedForInvalidate() throws Exception {
         String originalName = artist.getArtistName();
 
-        // create alternative context
-        final DataContext altContext = mirrorDataContext(context);
-
         // make sure we have a fully resolved copy of an artist object
         // in the second context
-        final Artist altArtist = (Artist) altContext.getGraphManager().getNode(
-                artist.getObjectId());
-        altContext.prepareForAccess(altArtist, null, false);
+        final Artist altArtist = context1.localObject(artist);
+        context1.prepareForAccess(altArtist, null, false);
         assertEquals(PersistenceState.COMMITTED, altArtist.getPersistenceState());
 
-        context.invalidateObjects(Collections.singletonList(artist));
+        context.invalidateObjects(artist);
 
         // original context
         assertEquals(PersistenceState.HOLLOW, artist.getPersistenceState());
-        assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(
-                artist.getObjectId()));
+        assertNull(context
+                .getObjectStore()
+                .getDataRowCache()
+                .getCachedSnapshot(artist.getObjectId()));
 
         // alternate context
         new ThreadedTestHelper() {
@@ -616,7 +602,7 @@ public class DataContextSharedCacheTest extends MultiContextCase {
             @Override
             protected void assertResult() throws Exception {
                 assertEquals(PersistenceState.HOLLOW, altArtist.getPersistenceState());
-                assertNull(altContext
+                assertNull(context1
                         .getObjectStore()
                         .getDataRowCache()
                         .getCachedSnapshot(altArtist.getObjectId()));
@@ -625,7 +611,7 @@ public class DataContextSharedCacheTest extends MultiContextCase {
 
         // resolve object
         assertEquals(originalName, altArtist.getArtistName());
-        DataRow altFreshSnapshot = altContext
+        DataRow altFreshSnapshot = context1
                 .getObjectStore()
                 .getDataRowCache()
                 .getCachedSnapshot(altArtist.getObjectId());
@@ -640,8 +626,10 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         assertEquals(PersistenceState.COMMITTED, artist.getPersistenceState());
 
         context.getObjectStore().getDataRowCache().forgetSnapshot(artist.getObjectId());
-        assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(
-                artist.getObjectId()));
+        assertNull(context
+                .getObjectStore()
+                .getDataRowCache()
+                .getCachedSnapshot(artist.getObjectId()));
 
         // modify object and try to save
         artist.setArtistName(newName);
@@ -665,8 +653,10 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         artist.setArtistName(newName);
 
         context.getObjectStore().getDataRowCache().forgetSnapshot(artist.getObjectId());
-        assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(
-                artist.getObjectId()));
+        assertNull(context
+                .getObjectStore()
+                .getDataRowCache()
+                .getCachedSnapshot(artist.getObjectId()));
 
         context.commitChanges();
 
@@ -690,8 +680,10 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         artist.setArtistName(newName);
 
         context.getObjectStore().getDataRowCache().forgetSnapshot(artist.getObjectId());
-        assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(
-                artist.getObjectId()));
+        assertNull(context
+                .getObjectStore()
+                .getDataRowCache()
+                .getCachedSnapshot(artist.getObjectId()));
 
         // now replace the row in the database
         String template = "UPDATE ARTIST SET ARTIST_NAME = #bind($newName) WHERE ARTIST_NAME = #bind($oldName)";
@@ -721,7 +713,7 @@ public class DataContextSharedCacheTest extends MultiContextCase {
         assertEquals(PersistenceState.COMMITTED, artist.getPersistenceState());
 
         // delete object PRIOR to killing the snapshot
-        context.deleteObject(artist);
+        context.deleteObjects(artist);
 
         context.getObjectStore().getDataRowCache().forgetSnapshot(id);
         assertNull(context.getObjectStore().getDataRowCache().getCachedSnapshot(id));
