@@ -18,9 +18,9 @@
  ****************************************************************/
 package org.apache.cayenne.ejbql;
 
-import org.apache.cayenne.unit.CayenneCase;
+import junit.framework.TestCase;
 
-public class EJBQLExpressionTest extends CayenneCase {
+public class EJBQLExpressionTest extends TestCase {
 
     public void testDbPath() {
 
@@ -30,4 +30,12 @@ public class EJBQLExpressionTest extends CayenneCase {
                 .parse("select p from Painting p WHERE db:p.toArtist.ARTIST_NAME = 'a'");
         assertNotNull(select);
     }
+
+    public void testEnumPath() {
+        EJBQLParser parser = EJBQLParserFactory.getParser();
+
+        EJBQLExpression select = parser
+                .parse("select p from Painting p WHERE p.toArtist.ARTIST_NAME = enum:org.apache.cayenne.ejbql.EJBQLEnum1.X");
+        assertNotNull(select);
+    }
 }
