@@ -38,4 +38,14 @@ public class MockRelationship extends Relationship {
 
     public void encodeAsXML(XMLEncoder encoder) {
     }
+
+    @Override
+    public Relationship getReverseRelationship() {
+        return null;
+    }
+
+    @Override
+    public boolean isMandatory() {
+        return false;
+    }
 }
