@@ -19,16 +19,20 @@
 package org.apache.cayenne.remote;
 
 import org.apache.cayenne.testdo.mt.ClientMtTable1;
+import org.apache.cayenne.unit.di.client.ClientCase;
+import org.apache.cayenne.unit.di.server.UseServerRuntime;
 
+@UseServerRuntime(ClientCase.MULTI_TIER_PROJECT)
 public class CayenneContextDeletionTest extends RemoteCayenneCase {
+
     public void testDeletion() {
-        ClientMtTable1 obj = context.newObject(ClientMtTable1.class);
-        context.commitChanges();
-        
-        context.deleteObject(obj);
-        
-        //now check that the object is unregistered
-        context.commitChanges();
-        assertNull(context.getGraphManager().getNode(obj.getObjectId()));
+        ClientMtTable1 object = clientContext.newObject(ClientMtTable1.class);
+        clientContext.commitChanges();
+
+        clientContext.deleteObjects(object);
+
+        // now check that the object is unregistered
+        clientContext.commitChanges();
+        assertNull(clientContext.getGraphManager().getNode(object.getObjectId()));
     }
 }
